use scan_central_db;

CREATE TABLE IF NOT EXISTS `sc_student`
(
    `id`                   bigint      NOT NULL AUTO_INCREMENT,
    `exam_id`              bigint      NOT NULL,
    `exam_number`          varchar(64) NOT NULL,
    `name`                 varchar(64) NOT NULL,
    `subject_code`         varchar(64) NOT NULL,
    `package_code`         varchar(64) DEFAULT NULL,
    `exam_site`            varchar(64) DEFAULT NULL,
    `exam_site_name`       varchar(64) DEFAULT NULL,
    `exam_room`            varchar(64) DEFAULT NULL,
    `seat_number`          varchar(16) DEFAULT NULL,
    `campus_name`          varchar(64) DEFAULT NULL,
    `campus_code`          varchar(64) DEFAULT NULL,
    `province`				varchar(64) DEFAULT NULL,
    `status`               varchar(16) NOT NULL,
    `omr_absent`           bit(1)      NOT NULL,
    `question_filled`      bit(1)      NOT NULL,
    `subjective_filled`    bit(1)      NOT NULL,
    `assigned`             bit(1)      NOT NULL,
    `absent_suspect`       bit(1)      NOT NULL,
    `incomplete`           bit(1)      NOT NULL,
    `card_number`          int         DEFAULT NULL,
    `device`               varchar(200) DEFAULT NULL,
    `paper_type`           varchar(16) NOT NULL,
    `exam_status`          varchar(32) DEFAULT NULL,
    `breach_code`          varchar(32) DEFAULT NULL,
    `file_upload_status`   varchar(16) DEFAULT NULL,
    `data_upload_status`   varchar(16) DEFAULT NULL,
    `check_mark`           bit(1)      NOT NULL,
    `assigned_check_count` int         DEFAULT NULL,
    `assigned_suspect`     bit(1)      NOT NULL,
    `image_check_status`   varchar(32) NOT NULL,
    `creator_id`           bigint      DEFAULT NULL,
    `updater_id`           bigint      DEFAULT NULL,
    `create_time`          bigint      DEFAULT NULL,
    `update_time`          bigint      DEFAULT NULL,
    PRIMARY KEY (`id`),
    UNIQUE KEY `exam_student_subject` (`exam_id`, `subject_code`, `exam_number`),
    KEY `exam_assigned` (`exam_id`, `assigned`, `assigned_check_count`),
    KEY `exam_image_check_status` (`exam_id`, `image_check_status`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4;

-- ----------------------------
-- Table structure for sc_answer_card
-- ----------------------------
CREATE TABLE IF NOT EXISTS `sc_answer_card`
(
    `exam_id`      bigint       NOT NULL,
    `number`       int          NOT NULL,
    `code`         varchar(128) DEFAULT NULL,
    `paper_count`  int          NOT NULL,
    `path`         varchar(255) NOT NULL,
    `md5`          varchar(32)  NOT NULL,
    `source`       varchar(16)  NOT NULL,
    `need_adapte`  bit(1)       NOT NULL,
    `single_page`  bit(1)       NOT NULL,
    `slice_config` text         NOT NULL,
    `dpi`          int          DEFAULT NULL,
    `slice_name`   text         DEFAULT NULL,
    `parameter`    text         DEFAULT NULL,
    `remark`       varchar(255) DEFAULT NULL,
    `creator_id`   bigint       DEFAULT NULL,
    `updater_id`   bigint       DEFAULT NULL,
    `create_time`  bigint       DEFAULT NULL,
    `update_time`  bigint       DEFAULT NULL,
    PRIMARY KEY (`exam_id`, `number`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4;

-- ----------------------------
-- Table structure for sc_exam
-- ----------------------------
CREATE TABLE IF NOT EXISTS `sc_exam`
(
    `id`                               bigint       NOT NULL AUTO_INCREMENT,
    `name`                             varchar(128) NOT NULL,
    `enable`                           bit(1)       NOT NULL,
    `mode`                             varchar(16)  NOT NULL,
    `scan_by_package`                  bit(1)       NOT NULL,
    `allow_unexist_paper`              bit(1)       NOT NULL,
    `answer_front_card_type`           int          NOT NULL,
    `enable_single_page_answer`        bit(1)       NOT NULL,
    `enable_upload`                    bit(1)       NOT NULL,
    `enable_sync_verify`               bit(1)       NOT NULL,
    `exam_number_fill_count`           int         DEFAULT NULL,
    `paper_type_barcode_content`       text        DEFAULT NULL,
    `absent_barcode_content`           varchar(16) DEFAULT NULL,
    `image_transfer_mode`              varchar(32) DEFAULT NULL,
    `image_check_ratio`                double      DEFAULT NULL,
    `image_check_order`                varchar(50) DEFAULT NULL,
    `scanner_assigned_max_count`       int         DEFAULT NULL,
    `scanner_assigned_verify_password` varchar(50) DEFAULT NULL,
    `year`                             int         DEFAULT NULL,
    `year_half`                        int         DEFAULT NULL,
    `scan_site`                        varchar(50) DEFAULT NULL,
    `data_sync_time`                   bigint      DEFAULT NULL,
    `card_sync_time`                   bigint      DEFAULT NULL,
    `creator_id`                       bigint      DEFAULT NULL,
    `updater_id`                       bigint      DEFAULT NULL,
    `create_time`                      bigint      DEFAULT NULL,
    `update_time`                      bigint      DEFAULT NULL,
    PRIMARY KEY (`id`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4;

-- ----------------------------
-- Table structure for sc_exam_summary
-- ----------------------------
CREATE TABLE IF NOT EXISTS `sc_exam_summary`
(
    `id`                          bigint  NOT NULL,
    `student_count`               int(11) NOT NULL,
    `subject_count`               int(11) NOT NULL,
    `batch_count`                 int(11) NOT NULL,
    `answer_scanned_count`        int(11) NOT NULL,
    `answer_unexist_count`        int(11) NOT NULL,
    `answer_manual_absent_count`  int(11) NOT NULL,
    `answer_omr_absent_count`     int(11) NOT NULL,
    `answer_absent_suspect_count` int(11) NOT NULL,
    `answer_incomplete_count`     int(11) NOT NULL,
    `answer_assigned_count`       int(11) NOT NULL,
    `answer_mismatch_count`       int(11) NOT NULL,
    `answer_batch_count`          int(11) NOT NULL,
    `package_total_count`         int(11) NOT NULL,
    `package_scanned_count`       int(11) NOT NULL,
    `student_answer_count`        int(11) NOT NULL,
    `upload_progress`             double  NOT NULL,
    PRIMARY KEY (`id`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4;

-- ----------------------------
-- Table structure for sc_omr_task
-- ----------------------------
CREATE TABLE IF NOT EXISTS `sc_omr_task`
(
    `id`           bigint      NOT NULL AUTO_INCREMENT,
    `exam_id`      bigint      NOT NULL,
    `group_id`     bigint      NOT NULL,
    `student_id`   bigint      NOT NULL,
    `paper_number` int         NOT NULL,
    `paper_id`     int         NOT NULL,
    `card_number`  int         NOT NULL,
    `status`       varchar(16) NOT NULL,
    `device`       varchar(200) DEFAULT NULL,
    `pages`        text        NOT NULL,
    `creator_id`   bigint      DEFAULT NULL,
    `updater_id`   bigint      DEFAULT NULL,
    `create_time`  bigint      DEFAULT NULL,
    `update_time`  bigint      DEFAULT NULL,
    PRIMARY KEY (`id`),
    UNIQUE KEY `group_paper` (`group_id`, `paper_id`),
    KEY `exam_status` (`exam_id`, `status`, `student_id`),
    KEY `group_student` (`group_id`, `student_id`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4;

-- ----------------------------
-- Table structure for sc_package_card
-- ----------------------------
CREATE TABLE IF NOT EXISTS `sc_package_card`
(
    `id`          bigint       NOT NULL AUTO_INCREMENT,
    `exam_id`     bigint       NOT NULL,
    `path`        varchar(255) NOT NULL,
    `md5`         varchar(32)  NOT NULL,
    `source`      varchar(16)  NOT NULL,
    `creator_id`  bigint DEFAULT NULL,
    `updater_id`  bigint DEFAULT NULL,
    `create_time` bigint DEFAULT NULL,
    `update_time` bigint DEFAULT NULL,
    PRIMARY KEY (`id`),
    KEY `exam` (`exam_id`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4;

-- ----------------------------
-- Table structure for sc_package_task
-- ----------------------------
CREATE TABLE IF NOT EXISTS `sc_package_task`
(
    `id`           bigint      NOT NULL AUTO_INCREMENT,
    `exam_id`      bigint      NOT NULL,
    `package_code` varchar(64) NOT NULL,
    `status`       varchar(16) NOT NULL,
    `device`       varchar(200) NOT NULL,
    `creator_id`   bigint DEFAULT NULL,
    `updater_id`   bigint DEFAULT NULL,
    `create_time`  bigint DEFAULT NULL,
    `update_time`  bigint DEFAULT NULL,
    PRIMARY KEY (`id`),
    KEY `exam_package` (`exam_id`, `package_code`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4;

-- ----------------------------
-- Table structure for sc_scanner
-- ----------------------------
CREATE TABLE IF NOT EXISTS `sc_scanner`
(
    `device`          varchar(200)  NOT NULL,
    `device_name`     varchar(200) NOT NULL,
    `last_login_time` bigint       NOT NULL,
    `create_time`     bigint DEFAULT NULL,
    `update_time`     bigint DEFAULT NULL,
    PRIMARY KEY (`device`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4;

-- ----------------------------
-- Table structure for ss_system_config
-- ----------------------------
CREATE TABLE IF NOT EXISTS `sc_system_config`
(
    `id`                   bigint      NOT NULL AUTO_INCREMENT,
    `scanner_enable_login` bit(1)      NOT NULL,
    `scanner_password`     varchar(16) NOT NULL,
    `client_version`       varchar(16)  DEFAULT NULL,
    `client_uri`           varchar(255) DEFAULT NULL,
    `client_md5`           varchar(32)  DEFAULT NULL,
    `client_update_time`   bigint       DEFAULT NULL,
    PRIMARY KEY (`id`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4;

CREATE TABLE IF NOT EXISTS `sc_package_result`
(
    `exam_id`      bigint      NOT NULL,
    `package_code` varchar(64) NOT NULL,
    `device`       varchar(200) NOT NULL,
    `path`         text        NOT NULL,
    `assigned`     bit(1)      NOT NULL,
    `creator_id`   bigint DEFAULT NULL,
    `updater_id`   bigint DEFAULT NULL,
    `create_time`  bigint DEFAULT NULL,
    `update_time`  bigint DEFAULT NULL,
    `upload_time`  bigint DEFAULT NULL,
    PRIMARY KEY (`exam_id`, `package_code`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4;

CREATE TABLE IF NOT EXISTS `sc_scanner_card`
(
    `exam_id`     bigint      NOT NULL,
    `device`      varchar(200) NOT NULL,
    `card_number` int         NOT NULL,
    `creator_id`  bigint DEFAULT NULL,
    `updater_id`  bigint DEFAULT NULL,
    `create_time` bigint DEFAULT NULL,
    `update_time` bigint DEFAULT NULL,
    PRIMARY KEY (`exam_id`, `device`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4;

CREATE TABLE IF NOT EXISTS `sc_batch`
(
    `id`                  bigint      NOT NULL AUTO_INCREMENT,
    `exam_id`             bigint      NOT NULL,
    `device`              varchar(200) NOT NULL,
    `package_code`        varchar(64) DEFAULT NULL,
    `subject_code`        varchar(64) DEFAULT NULL,
    `scan_count`          int         NOT NULL,
    `assigned_count`      int         NOT NULL,
    `status`              varchar(16) NOT NULL,
    `verify_status`       varchar(16) DEFAULT NULL,
    `check_status`        varchar(16) DEFAULT NULL,
    `check_image_user_id` bigint      DEFAULT NULL,
    `check_image_time`    bigint      DEFAULT NULL,
    `creator_id`          bigint      DEFAULT NULL,
    `updater_id`          bigint      DEFAULT NULL,
    `create_time`         bigint      DEFAULT NULL,
    `update_time`         bigint      DEFAULT NULL,
    PRIMARY KEY (`id`),
    KEY `exam_verify_status` (`exam_id`, `verify_status`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4;

CREATE TABLE IF NOT EXISTS `sc_batch_paper`
(
    `batch_id`     bigint(20) NOT NULL,
    `student_id`   bigint(20) NOT NULL,
    `paper_number` int(11)    NOT NULL,
    `paper_id`     bigint(20) NOT NULL,
    `card_number`  int(11)    NOT NULL,
    `assigned`     bit(1)     NOT NULL,
    `need_check`   bit(1)     NOT NULL,
    `creator_id`   bigint DEFAULT NULL,
    `updater_id`   bigint DEFAULT NULL,
    `create_time`  bigint DEFAULT NULL,
    `update_time`  bigint DEFAULT NULL,
    PRIMARY KEY (`batch_id`, `student_id`, `paper_number`),
    UNIQUE KEY `paper_id` (`paper_id`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4;

CREATE TABLE IF NOT EXISTS `sc_paper`
(
    `id`                bigint       NOT NULL AUTO_INCREMENT,
    `exam_id`           bigint       NOT NULL,
    `card_number`       int          NOT NULL,
    `number`            int          NOT NULL,
    `page_count`        int          NOT NULL,
    `mismatch`          bit(1)       NOT NULL,
    `assigned`          bit(1)       NOT NULL,
    `question_filled`   bit(1)       NOT NULL,
    `subjective_filled` bit(1)       NOT NULL,
    `omr_exam_number`   varchar(255) NOT NULL,
    `creator_id`        bigint DEFAULT NULL,
    `updater_id`        bigint DEFAULT NULL,
    `create_time`       bigint DEFAULT NULL,
    `update_time`       bigint DEFAULT NULL,
    PRIMARY KEY (`id`),
    KEY `mismatch` (`exam_id`, `mismatch`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4;

CREATE TABLE IF NOT EXISTS `sc_paper_page`
(
    `paper_id`    bigint       NOT NULL,
    `page_index`  int          NOT NULL,
    `absent`      text     DEFAULT NULL,
    `breach`      text     DEFAULT NULL,
    `paper_type`  text     DEFAULT NULL,
    `question`    text     DEFAULT NULL,
    `selective`   text     DEFAULT NULL,
    `sheet_path`  varchar(255) NOT NULL,
    `slice_path`  text     DEFAULT NULL,
    `recog_data`  longtext DEFAULT NULL,
    `creator_id`  bigint   DEFAULT NULL,
    `updater_id`  bigint   DEFAULT NULL,
    `create_time` bigint   DEFAULT NULL,
    `update_time` bigint   DEFAULT NULL,
    PRIMARY KEY (`paper_id`, `page_index`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4;

CREATE TABLE IF NOT EXISTS `sc_paper_structure`
(
    `id`              bigint      NOT NULL AUTO_INCREMENT,
    `exam_id`         bigint      NOT NULL,
    `card_number`     int         NOT NULL,
    `paper_number`    int         NOT NULL,
    `page_index`      int         NOT NULL,
    `field`           varchar(16) NOT NULL,
    `question_number` int         NOT NULL,
    `single`          bit(1)      NOT NULL,
    `creator_id`      bigint DEFAULT NULL,
    `updater_id`      bigint DEFAULT NULL,
    `create_time`     bigint DEFAULT NULL,
    `update_time`     bigint DEFAULT NULL,
    PRIMARY KEY (`id`),
    UNIQUE KEY `IDX_PAPER_STRUCTURE_001` (`exam_id`, `card_number`, `paper_number`, `page_index`, `field`,
                                          `question_number`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4;

CREATE TABLE IF NOT EXISTS `sc_question`
(
    `id`           int(11)      NOT NULL AUTO_INCREMENT COMMENT '主键',
    `subject_code` varchar(32)  NOT NULL COMMENT '科目代码',
    `objective`    tinyint(1)   NOT NULL COMMENT '是否客观题',
    `main_number`  int(11)      NOT NULL COMMENT '大题号',
    `sub_number`   varchar(32)  NOT NULL COMMENT '小题号',
    `main_title`   varchar(128) NOT NULL COMMENT '大题名称',
    `creator_id`   bigint DEFAULT NULL,
    `updater_id`   bigint DEFAULT NULL,
    `create_time`  bigint DEFAULT NULL,
    `update_time`  bigint DEFAULT NULL,
    PRIMARY KEY (`id`),
    UNIQUE KEY `index1` (`subject_code`,`main_number`, `sub_number`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4 COMMENT ='小题信息表';


CREATE TABLE IF NOT EXISTS `sc_omr_group`
(
    `id`          bigint      NOT NULL AUTO_INCREMENT,
    `exam_id`     bigint      NOT NULL,
    `subject_code`       varchar(64) NOT NULL,
    `fixed`       bit(1)      NOT NULL,
    `stage`       varchar(16) NOT NULL,
    `conditions`  text        NOT NULL,
    `total_count` int         NOT NULL,
    `creator_id`  bigint DEFAULT NULL,
    `updater_id`  bigint DEFAULT NULL,
    `create_time` bigint DEFAULT NULL,
    `update_time` bigint DEFAULT NULL,
    PRIMARY KEY (`id`),
    KEY `exam_fixed` (`exam_id`,`subject_code` , `fixed`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4;


CREATE TABLE IF NOT EXISTS `sc_refix_batch`
(
    `id`           bigint NOT NULL AUTO_INCREMENT,
    `exam_id`      bigint NOT NULL,
    `card_number`  int    NOT NULL,
    `conditions`   text   NOT NULL,
    `update_slice` bit(1) NOT NULL,
    `total_count`  int    NOT NULL,
    `sucess_count` int    NOT NULL,
    `finished`     bit(1) NOT NULL,
    `creator_id`   bigint DEFAULT NULL,
    `updater_id`   bigint DEFAULT NULL,
    `create_time`  bigint DEFAULT NULL,
    `update_time`  bigint DEFAULT NULL,
    PRIMARY KEY (`id`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4;

CREATE TABLE IF NOT EXISTS `sc_student_paper`
(
    `student_id`   bigint NOT NULL,
    `paper_number` int    NOT NULL,
    `paper_id`     bigint NOT NULL,
    `create_time`  bigint DEFAULT NULL,
    `update_time`  bigint DEFAULT NULL,
    PRIMARY KEY (`student_id`, `paper_number`),
    UNIQUE KEY `paper_id` (`paper_id`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4;

CREATE TABLE IF NOT EXISTS `sc_subject`
(
    `exam_id`                    bigint      NOT NULL,
    `code`                       varchar(64) NOT NULL,
    `name`                       varchar(64) NOT NULL,
    `paper_type_barcode_content` text   DEFAULT NULL,
    `creator_id`                 bigint DEFAULT NULL,
    `updater_id`                 bigint DEFAULT NULL,
    `create_time`                bigint DEFAULT NULL,
    `update_time`                bigint DEFAULT NULL,
    PRIMARY KEY (`exam_id`, `code`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4;

CREATE TABLE IF NOT EXISTS `sc_file_property`
(
    `path`        varchar(255) NOT NULL,
    `md5`         varchar(32)  NOT NULL,
    `exam_id`     bigint       NOT NULL,
    `file_size`   bigint       NOT NULL,
    `create_time` bigint DEFAULT NULL,
    `update_time` bigint DEFAULT NULL,
    PRIMARY KEY (`path`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4;

CREATE TABLE IF NOT EXISTS `sc_user`
(
    `id`          int(11)     NOT NULL AUTO_INCREMENT,
    `login_name`  varchar(64) NOT NULL,
    `name`        varchar(64) NOT NULL,
    `password`    varchar(64) NOT NULL,
    `role`        varchar(16) NOT NULL,
    `enable`      tinyint(1)  NOT NULL,
    `device`      varchar(200) DEFAULT NULL,
    `creator_id`  bigint      DEFAULT NULL,
    `updater_id`  bigint      DEFAULT NULL,
    `create_time` bigint      DEFAULT NULL,
    `update_time` bigint      DEFAULT NULL,
    PRIMARY KEY (`id`),
    UNIQUE KEY `login_name` (`login_name`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4;

CREATE TABLE IF NOT EXISTS `sc_adapte_file`
(
    `exam_id`     bigint       NOT NULL,
    `card_number` int          NOT NULL,
    `role`        varchar(16)  NOT NULL,
    `device`      varchar(200)  NOT NULL,
    `uri`         varchar(255) NOT NULL,
    `md5`         varchar(32)  NOT NULL,
    `dpi`         int    DEFAULT NULL,
    `create_time` bigint DEFAULT NULL,
    `update_time` bigint DEFAULT NULL,
    PRIMARY KEY (`exam_id`, `card_number`, `role`, `device`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4;

CREATE TABLE IF NOT EXISTS `sc_student_answer_task`
(
    `id`          bigint      NOT NULL AUTO_INCREMENT,
    `exam_id`     bigint      NOT NULL,
    `group_id`    bigint      NOT NULL,
    `student_id`  bigint      NOT NULL,
    `status`      varchar(16) NOT NULL,
    `device`      varchar(200) DEFAULT NULL,
    `creator_id`  bigint      DEFAULT NULL,
    `updater_id`  bigint      DEFAULT NULL,
    `create_time` bigint      DEFAULT NULL,
    `update_time` bigint      DEFAULT NULL,
    PRIMARY KEY (`id`),
    UNIQUE KEY `group_student` (`group_id`, `student_id`),
    KEY `exam_status` (`exam_id`, `student_id`, `status`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4;

CREATE TABLE IF NOT EXISTS `sc_student_answer_group`
(
    `id`          bigint NOT NULL AUTO_INCREMENT,
    `exam_id`     bigint NOT NULL,
    `conditions`  text   NOT NULL,
    `total_count` int    NOT NULL,
    `creator_id`  bigint DEFAULT NULL,
    `updater_id`  bigint DEFAULT NULL,
    `create_time` bigint DEFAULT NULL,
    `update_time` bigint DEFAULT NULL,
    PRIMARY KEY (`id`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4;

CREATE TABLE IF NOT EXISTS `sc_answer_card_subject`
(
    `exam_id`      bigint      NOT NULL,
    `card_number`  int         NOT NULL,
    `subject_code` varchar(64) NOT NULL,
    `create_time`  bigint DEFAULT NULL,
    `update_time`  bigint DEFAULT NULL,
    PRIMARY KEY (`exam_id`, `card_number`, `subject_code`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4;

-- ----------------------------
-- Table structure for sc_assigned_check_history
-- ----------------------------
CREATE TABLE IF NOT EXISTS `sc_assigned_check_history`
(
    `id`          bigint NOT NULL AUTO_INCREMENT,
    `exam_id`     bigint NOT NULL,
    `subject_code` varchar(64) NOT NULL,
    `student_id`  bigint NOT NULL,
    `user_id`     bigint NOT NULL,
    `creator_id`  bigint DEFAULT NULL,
    `updater_id`  bigint DEFAULT NULL,
    `create_time` bigint DEFAULT NULL,
    `update_time` bigint DEFAULT NULL,
    PRIMARY KEY (`id`),
    UNIQUE KEY `student_user` (`student_id`, `user_id`),
    KEY `student` (`student_id`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4;

CREATE TABLE IF NOT EXISTS `sc_mark_site`
(
    `id`           bigint       NOT NULL AUTO_INCREMENT,
    `exam_id`      bigint       NOT NULL,
    `subject_code` varchar(100) NOT NULL,
    `paper_type`   varchar(100) NOT NULL,
    `odd_number`   varchar(100) NOT NULL,
    `even_number`  varchar(100) NOT NULL,
    `creator_id`   bigint DEFAULT NULL,
    `updater_id`   bigint DEFAULT NULL,
    `create_time`  bigint DEFAULT NULL,
    `update_time`  bigint DEFAULT NULL,
    PRIMARY KEY (`id`),
    UNIQUE KEY `exam_subject_paper` (`exam_id`, `subject_code`, `paper_type`)
) ENGINE = InnoDB
  DEFAULT CHARSET = utf8mb4;
  
INSERT INTO `sc_system_config` (`id`, `scanner_enable_login`, `scanner_password`) 
VALUES (1,1, '');

 
INSERT INTO `sc_user` (`login_name`, `name`, `password`, `role`, `enable`, `create_time`, `update_time`) 
VALUES ('admin1', '学校管理员1', '123456', 'SCHOOL_ADMIN', 1, now(), now());
INSERT INTO `sc_user` (`login_name`, `name`, `password`, `role`, `enable`, `create_time`, `update_time`) 
VALUES ('admin2', '学校管理员2', '123456', 'SCHOOL_ADMIN', 1, now(), now());
INSERT INTO `sc_user` (`login_name`, `name`, `password`, `role`, `enable`, `create_time`, `update_time`) 
VALUES ('admin3', '学校管理员3', '123456', 'SCHOOL_ADMIN', 1, now(), now());
INSERT INTO `sc_user` (`login_name`, `name`, `password`, `role`, `enable`, `create_time`, `update_time`) 
VALUES ('admin4', '学校管理员4', '123456', 'SCHOOL_ADMIN', 1, now(), now());
INSERT INTO `sc_user` (`login_name`, `name`, `password`, `role`, `enable`, `create_time`, `update_time`) 
VALUES ('admin5', '学校管理员5', '123456', 'SCHOOL_ADMIN', 1, now(), now()); 
  

INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('1',0,1,'1','作文');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('1',1,2,'1','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('1',1,2,'2','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('1',1,2,'3','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('1',1,2,'4','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('1',1,2,'5','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('1',1,2,'6','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('1',1,2,'7','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('1',1,2,'8','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('1',1,2,'9','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('1',1,2,'10','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('1',1,2,'11','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('1',1,2,'12','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('1',1,2,'13','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('1',1,2,'14','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('1',1,2,'15','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('1',1,2,'16','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('1',1,2,'17','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('1',1,2,'18','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('1',1,2,'19','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('1',1,2,'20','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('1',1,2,'21','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('1',1,2,'22','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('1',1,2,'23','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('1',1,2,'24','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('1',1,2,'25','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('1',1,3,'26','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('1',1,3,'27','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('1',1,3,'28','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('1',1,3,'29','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('1',1,3,'30','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('1',1,3,'31','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('1',1,3,'32','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('1',1,3,'33','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('1',1,3,'34','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('1',1,3,'35','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('1',1,3,'36','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('1',1,3,'37','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('1',1,3,'38','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('1',1,3,'39','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('1',1,3,'40','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('1',1,3,'41','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('1',1,3,'42','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('1',1,3,'43','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('1',1,3,'44','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('1',1,3,'45','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('1',1,3,'46','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('1',1,3,'47','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('1',1,3,'48','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('1',1,3,'49','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('1',1,3,'50','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('1',1,3,'51','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('1',1,3,'52','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('1',1,3,'53','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('1',1,3,'54','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('1',1,3,'55','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('1',0,4,'1','翻译');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('2',0,1,'1','作文');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('2',1,2,'1','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('2',1,2,'2','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('2',1,2,'3','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('2',1,2,'4','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('2',1,2,'5','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('2',1,2,'6','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('2',1,2,'7','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('2',1,2,'8','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('2',1,2,'9','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('2',1,2,'10','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('2',1,2,'11','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('2',1,2,'12','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('2',1,2,'13','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('2',1,2,'14','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('2',1,2,'15','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('2',1,2,'16','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('2',1,2,'17','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('2',1,2,'18','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('2',1,2,'19','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('2',1,2,'20','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('2',1,2,'21','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('2',1,2,'22','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('2',1,2,'23','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('2',1,2,'24','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('2',1,2,'25','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('2',1,3,'26','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('2',1,3,'27','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('2',1,3,'28','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('2',1,3,'29','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('2',1,3,'30','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('2',1,3,'31','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('2',1,3,'32','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('2',1,3,'33','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('2',1,3,'34','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('2',1,3,'35','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('2',1,3,'36','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('2',1,3,'37','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('2',1,3,'38','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('2',1,3,'39','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('2',1,3,'40','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('2',1,3,'41','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('2',1,3,'42','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('2',1,3,'43','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('2',1,3,'44','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('2',1,3,'45','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('2',1,3,'46','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('2',1,3,'47','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('2',1,3,'48','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('2',1,3,'49','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('2',1,3,'50','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('2',1,3,'51','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('2',1,3,'52','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('2',1,3,'53','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('2',1,3,'54','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('2',1,3,'55','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('2',0,4,'1','翻译');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,1,'1','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,1,'2','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,1,'3','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,1,'4','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,1,'5','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,1,'6','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,1,'7','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,1,'8','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,1,'9','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,1,'10','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,1,'11','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,1,'12','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,1,'13','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,1,'14','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,1,'15','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,2,'16','文字和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,2,'17','文字和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,2,'18','文字和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,2,'19','文字和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,2,'20','文字和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,2,'21','文字和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,2,'22','文字和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,2,'23','文字和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,2,'24','文字和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,2,'25','文字和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,2,'26','文字和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,2,'27','文字和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,2,'28','文字和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,2,'29','文字和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,2,'30','文字和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,2,'31','文字和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,2,'32','文字和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,2,'33','文字和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,2,'34','文字和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,2,'35','文字和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,2,'36','文字和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,2,'37','文字和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,2,'38','文字和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,2,'39','文字和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,2,'40','文字和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,2,'41','文字和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,2,'42','文字和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,2,'43','文字和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,2,'44','文字和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,2,'45','文字和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,3,'46','文法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,3,'47','文法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,3,'48','文法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,3,'49','文法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,3,'50','文法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,3,'51','文法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,3,'52','文法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,3,'53','文法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,3,'54','文法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,3,'55','文法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,3,'56','文法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,3,'57','文法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,3,'58','文法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,3,'59','文法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,3,'60','文法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,3,'61','文法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,3,'62','文法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,3,'63','文法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,3,'64','文法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,3,'65','文法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,4,'66','讲解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,4,'67','讲解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,4,'68','讲解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,4,'69','讲解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,4,'70','讲解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,4,'71','讲解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,4,'72','讲解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,4,'73','讲解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,4,'74','讲解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,4,'75','讲解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,4,'76','讲解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,4,'77','讲解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,4,'78','讲解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,4,'79','讲解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,4,'80','讲解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,4,'81','讲解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,4,'82','讲解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,4,'83','讲解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,4,'84','讲解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',1,4,'85','讲解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',0,5,'1','翻译和作文');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',0,5,'2','翻译和作文');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('3',0,5,'3','翻译和作文');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,1,'1','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,1,'2','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,1,'3','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,1,'4','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,1,'5','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,1,'6','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,1,'7','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,1,'8','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,1,'9','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,1,'10','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,1,'11','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,1,'12','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,1,'13','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,1,'14','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,1,'15','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,1,'16','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,1,'17','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,1,'18','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,1,'19','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,1,'20','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,1,'21','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,1,'22','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,1,'23','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,1,'24','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,1,'25','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,2,'26','词汇和文法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,2,'27','词汇和文法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,2,'28','词汇和文法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,2,'29','词汇和文法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,2,'30','词汇和文法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,2,'31','词汇和文法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,2,'32','词汇和文法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,2,'33','词汇和文法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,2,'34','词汇和文法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,2,'35','词汇和文法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,2,'36','词汇和文法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,2,'37','词汇和文法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,2,'38','词汇和文法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,2,'39','词汇和文法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,2,'40','词汇和文法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,2,'41','词汇和文法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,2,'42','词汇和文法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,2,'43','词汇和文法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,2,'44','词汇和文法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,2,'45','词汇和文法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,3,'46','讲解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,3,'47','讲解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,3,'48','讲解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,3,'49','讲解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,3,'50','讲解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,3,'51','讲解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,3,'52','讲解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,3,'53','讲解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,3,'54','讲解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,3,'55','讲解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,3,'56','讲解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,3,'57','讲解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,3,'58','讲解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,3,'59','讲解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,3,'60','讲解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,3,'61','讲解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,3,'62','讲解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,3,'63','讲解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,3,'64','讲解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,3,'65','讲解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,3,'66','讲解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,3,'67','讲解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',1,3,'68','讲解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',0,4,'1','和文中译');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',0,4,'2','和文中译');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('4',0,5,'1','作文');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,1,'1','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,1,'2','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,1,'3','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,1,'4','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,1,'5','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,1,'6','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,1,'7','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,1,'8','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,1,'9','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,1,'10','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,1,'11','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,1,'12','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,1,'13','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,1,'14','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,1,'15','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,1,'16','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,1,'17','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,1,'18','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,1,'19','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,1,'20','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,2,'21','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,2,'22','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,2,'23','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,2,'24','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,2,'25','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,2,'26','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,2,'27','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,2,'28','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,2,'29','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,2,'30','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,2,'31','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,2,'32','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,2,'33','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,2,'34','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,2,'35','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,2,'36','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,2,'37','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,2,'38','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,2,'39','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,3,'40','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,3,'41','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,3,'42','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,3,'43','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,3,'44','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,3,'45','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,3,'46','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,3,'47','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,3,'48','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,3,'49','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,3,'50','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,3,'51','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,3,'52','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,3,'53','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,3,'54','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,3,'55','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,3,'56','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,3,'57','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,3,'58','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,3,'59','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,3,'60','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,3,'61','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,3,'62','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,3,'63','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,3,'64','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,3,'65','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,3,'66','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',1,3,'67','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',0,4,'68','德译中');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',0,4,'69','德译中');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',0,4,'70','德译中');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',0,5,'71','书面表达');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',0,5,'72','书面表达');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',0,5,'73','书面表达');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',0,5,'74','书面表达');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',0,5,'75','书面表达');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('5',0,5,'76','书面表达');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',1,1,'1','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',1,1,'2','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',1,1,'3','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',1,1,'4','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',1,1,'5','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',1,1,'6','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',1,1,'7','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',1,1,'8','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',1,1,'9','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',1,1,'10','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',1,1,'11','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',1,1,'12','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',1,1,'13','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',1,1,'14','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',1,1,'15','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',1,1,'16','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',1,1,'17','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',1,1,'18','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',1,1,'19','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',1,1,'20','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',1,2,'21','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',1,2,'22','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',1,2,'23','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',1,2,'24','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',1,2,'25','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',1,2,'26','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',1,2,'27','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',1,2,'28','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',1,2,'29','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',1,2,'30','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',1,2,'31','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',1,2,'32','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',1,2,'33','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',1,2,'34','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',1,2,'35','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',1,2,'36','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',1,2,'37','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',1,2,'38','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',1,2,'39','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',1,2,'40','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',1,3,'41','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',1,3,'42','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',1,3,'43','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',1,3,'44','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',1,3,'45','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',1,3,'46','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',1,3,'47','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',1,3,'48','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',1,3,'49','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',1,3,'50','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',1,3,'51','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',1,3,'52','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',1,3,'53','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',1,3,'54','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',1,3,'55','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',1,3,'56','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',1,3,'57','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',1,3,'58','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',1,3,'59','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',1,3,'60','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',0,4,'61','德译中');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',0,4,'62','德译中');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('6',0,5,'63','书面表达');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,1,'1','审计');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,1,'2','审计');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,1,'3','审计');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,1,'4','审计');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,1,'5','审计');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,1,'6','审计');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,1,'7','审计');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,1,'8','审计');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,1,'9','审计');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,1,'10','审计');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,1,'11','审计');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,1,'12','审计');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,1,'13','审计');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,1,'14','审计');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,1,'15','审计');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,2,'16','词汇和语法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,2,'17','词汇和语法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,2,'18','词汇和语法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,2,'19','词汇和语法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,2,'20','词汇和语法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,2,'21','词汇和语法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,2,'22','词汇和语法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,2,'23','词汇和语法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,2,'24','词汇和语法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,2,'25','词汇和语法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,2,'26','词汇和语法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,2,'27','词汇和语法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,2,'28','词汇和语法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,2,'29','词汇和语法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,2,'30','词汇和语法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,2,'31','词汇和语法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,2,'32','词汇和语法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,2,'33','词汇和语法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,2,'34','词汇和语法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,2,'35','词汇和语法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,2,'36','词汇和语法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,2,'37','词汇和语法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,2,'38','词汇和语法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,2,'39','词汇和语法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,2,'40','词汇和语法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,2,'41','词汇和语法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,2,'42','词汇和语法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,2,'43','词汇和语法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,2,'44','词汇和语法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,2,'45','词汇和语法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,2,'46','词汇和语法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,2,'47','词汇和语法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,2,'48','词汇和语法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,2,'49','词汇和语法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,2,'50','词汇和语法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,3,'51','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,3,'52','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,3,'53','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,3,'54','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,3,'55','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,3,'56','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,3,'57','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,3,'58','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,3,'59','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,3,'60','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,3,'61','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,3,'62','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,3,'63','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,3,'64','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',1,3,'65','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',0,4,'66','翻译');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',0,4,'67','翻译');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('7',0,5,'68','作文');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('8',1,1,'1','审计');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('8',1,1,'2','审计');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('8',1,1,'3','审计');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('8',1,1,'4','审计');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('8',1,1,'5','审计');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('8',1,1,'6','审计');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('8',1,1,'7','审计');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('8',1,1,'8','审计');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('8',1,1,'9','审计');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('8',1,1,'10','审计');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('8',1,1,'11','审计');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('8',1,1,'12','审计');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('8',1,1,'13','审计');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('8',1,1,'14','审计');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('8',1,1,'15','审计');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('8',1,2,'16','词汇和语法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('8',1,2,'17','词汇和语法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('8',1,2,'18','词汇和语法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('8',1,2,'19','词汇和语法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('8',1,2,'20','词汇和语法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('8',1,2,'21','词汇和语法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('8',1,2,'22','词汇和语法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('8',1,2,'23','词汇和语法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('8',1,2,'24','词汇和语法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('8',1,2,'25','词汇和语法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('8',1,2,'26','词汇和语法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('8',1,2,'27','词汇和语法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('8',1,2,'28','词汇和语法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('8',1,2,'29','词汇和语法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('8',1,2,'30','词汇和语法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('8',1,2,'31','词汇和语法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('8',1,2,'32','词汇和语法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('8',1,2,'33','词汇和语法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('8',1,2,'34','词汇和语法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('8',1,2,'35','词汇和语法');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('8',1,3,'36','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('8',1,3,'37','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('8',1,3,'38','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('8',1,3,'39','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('8',1,3,'40','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('8',1,3,'41','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('8',1,3,'42','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('8',1,3,'43','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('8',1,3,'44','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('8',1,3,'45','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('8',1,3,'46','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('8',1,3,'47','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('8',1,3,'48','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('8',1,3,'49','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('8',1,3,'50','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('8',0,4,'51','翻译');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('8',0,4,'52','翻译');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('8',0,5,'53','作文');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('9',1,1,'1','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('9',1,1,'2','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('9',1,1,'3','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('9',1,1,'4','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('9',1,1,'5','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('9',1,1,'6','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('9',1,1,'7','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('9',1,1,'8','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('9',1,1,'9','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('9',1,1,'10','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('9',1,1,'11','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('9',1,1,'12','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('9',1,1,'13','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('9',1,1,'14','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('9',1,1,'15','听力');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('9',1,2,'16','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('9',1,2,'17','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('9',1,2,'18','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('9',1,2,'19','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('9',1,2,'20','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('9',1,2,'21','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('9',1,2,'22','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('9',1,2,'23','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('9',1,2,'24','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('9',1,2,'25','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('9',1,2,'26','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('9',1,2,'27','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('9',1,2,'28','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('9',1,2,'29','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('9',1,2,'30','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('9',1,2,'31','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('9',1,2,'32','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('9',1,2,'33','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('9',1,2,'34','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('9',1,2,'35','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('9',1,2,'36','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('9',1,2,'37','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('9',1,2,'38','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('9',1,2,'39','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('9',1,2,'40','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('9',1,2,'41','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('9',1,2,'42','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('9',1,2,'43','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('9',1,2,'44','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('9',1,2,'45','语法和词汇');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('9',1,3,'46','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('9',1,3,'47','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('9',1,3,'48','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('9',1,3,'49','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('9',1,3,'50','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('9',1,3,'51','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('9',1,3,'52','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('9',1,3,'53','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('9',1,3,'54','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('9',1,3,'55','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('9',1,3,'56','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('9',1,3,'57','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('9',1,3,'58','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('9',1,3,'59','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('9',1,3,'60','阅读理解');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('9',0,4,'1','翻译');
INSERT INTO `sc_question` (`subject_code`,`objective`,`main_number`, `sub_number`, `main_title`) VALUES ('9',0,5,'1','作文');
