package cn.com.qmth.scancentral.bean.answersave;

import cn.com.qmth.scancentral.entity.AnswerCardEntity;
import cn.com.qmth.scancentral.entity.PaperPageEntity;
import com.qmth.boot.core.exception.ParameterException;

import javax.validation.Valid;
import javax.validation.constraints.Min;
import javax.validation.constraints.NotNull;
import javax.validation.constraints.Size;
import java.util.ArrayList;
import java.util.List;

public class AnswerPaper {

    @NotNull(message = "paper.number不能为空")
    @Min(value = 1, message = "paper.number不能小于1")
    private Integer number;

    @NotNull(message = "paper.assigned不能为空")
    private Boolean assigned;

    @NotNull(message = "paper.subjectiveFilled不能为空")
    private Boolean subjectiveFilled;

    @NotNull(message = "paper.omrExamNumber不能为空")
    private String omrExamNumber;

    @Valid
    @NotNull(message = "paper.pages不能为空")
    @Size(min = 1, message = "paper.pages不能为空")
    private List<AnswerPage> pages;

    public Integer getNumber() {
        return number;
    }

    public void setNumber(Integer number) {
        this.number = number;
    }

    public Boolean getAssigned() {
        return assigned;
    }

    public void setAssigned(Boolean assigned) {
        this.assigned = assigned;
    }

    public List<AnswerPage> getPages() {
        return pages;
    }

    public void setPages(List<AnswerPage> pages) {
        this.pages = pages;
    }

    public Boolean getSubjectiveFilled() {
        return subjectiveFilled;
    }

    public void setSubjectiveFilled(Boolean subjectiveFilled) {
        this.subjectiveFilled = subjectiveFilled;
    }

    public String getOmrExamNumber() {
        return omrExamNumber;
    }

    public void setOmrExamNumber(String omrExamNumber) {
        this.omrExamNumber = omrExamNumber;
    }

    /**
     * 根据卡格式定义验证页数
     *
     * @param answerCard
     */
    public void answerCardValidate(AnswerCardEntity answerCard) {
        if (answerCard.getSinglePage() && getPages().size() != 1) {
            throw new ParameterException("卡格式页数不一致");
        }
        if (!answerCard.getSinglePage() && getPages().size() != 2) {
            throw new ParameterException("卡格式页数不一致");
        }
    }

    public List<PaperPageEntity> buildPageList(List<String> barcodeContents) {
        List<PaperPageEntity> list = new ArrayList<>();
        for (AnswerPage answerPage : pages) {
            PaperPageEntity page = new PaperPageEntity();
            page.setPageIndex(answerPage.getIndex());
            page.setSheetPath(answerPage.getSheetUri());
            page.setSlicePath(answerPage.getSliceUri());
            page.setAbsent(answerPage.getAbsent());
            page.setBreach(answerPage.getBreach());
            page.setPaperType(answerPage.getPaperType());
            page.setQuestion(answerPage.getQuestion());
            page.setSelective(answerPage.getSelective());
            page.setRecogData(answerPage.getRecogData());
            page.checkPaperType(barcodeContents);
            list.add(page);
        }
        return list;
    }

}
