package cn.com.qmth.scancentral.controller;

import cn.com.qmth.scancentral.bean.User;
import cn.com.qmth.scancentral.config.SysProperty;
import cn.com.qmth.scancentral.entity.AnswerCardEntity;
import cn.com.qmth.scancentral.entity.ExamEntity;
import cn.com.qmth.scancentral.entity.SystemConfigEntity;
import cn.com.qmth.scancentral.enums.Role;
import cn.com.qmth.scancentral.exception.ParameterExceptions;
import cn.com.qmth.scancentral.service.*;
import cn.com.qmth.scancentral.service.impl.SystemConfigServiceImpl;
import cn.com.qmth.scancentral.vo.ActiveTimeVo;
import cn.com.qmth.scancentral.vo.AnswerCardVo;
import cn.com.qmth.scancentral.vo.apistatus.ApiStatusClient;
import cn.com.qmth.scancentral.vo.apistatus.ApiStatusVo;
import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.boot.api.utils.RequestUtil;
import com.qmth.boot.core.exception.ParameterException;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
@Api(tags = "系统接口")
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX)
@Aac(strict = false, auth = true)
public class SystemController extends BaseController {

    @Autowired
    private SessionService sessionService;

    @Autowired
    private SysProperty sysProperty;

    @Autowired
    private AnswerCardService answerCardService;

    @Autowired
    private ExamService examService;

    @Autowired
    private AdapteFileService adapteFileService;

    @Autowired
    private AnswerCardSubjectService answerCardSubjectService;

    @Autowired
    private SystemConfigServiceImpl systemConfigService;

    @ApiOperation(value = "心跳接口")
    @PostMapping("/sys/heartbeat")
    public ActiveTimeVo heartbeat() {
        User user = getAccessUser();
        ActiveTimeVo vo = new ActiveTimeVo();
        vo.setActiveTime(sessionService.updateUserSession(user));
        return vo;
    }

    @Aac(auth = false)
    @ApiOperation(value = "服务端状态")
    @RequestMapping(value = "/sys/status", method = {RequestMethod.GET, RequestMethod.POST})
    public ApiStatusVo status(HttpServletRequest request) {
        ApiStatusVo vo = new ApiStatusVo();
        vo.setFileUriPrefix(sysProperty.getFileUriPrefix());
        vo.setVersion(sysProperty.getVersion());
        vo.setTime(System.currentTimeMillis());
        vo.setClientIP(RequestUtil.getIpAddress(request));

        ApiStatusClient client = new ApiStatusClient();
        SystemConfigEntity config = systemConfigService.find();
        if (config != null) {
            client.setVersion(config.getClientVersion());
            client.setMd5(config.getClientMd5());
            client.setUpdateTime(config.getClientUpdateTime());
            client.setUri(config.getClientUri());
        }
        vo.setClient(client);
        return vo;
    }

    @Aac(auth = false)
    @ApiOperation(value = "用户角色类型")
    @PostMapping("/sys/role/list")
    public List<Map<String, String>> roles() {
        List<Map<String, String>> roles = new ArrayList<>();
        for (Role r : Role.values()) {
            Map<String, String> map = new HashMap<>();
            map.put("roleCode", r.name());
            map.put("roleName", r.getName());
            roles.add(map);
        }
        return roles;
    }

    @ApiOperation(value = "答题卡卡格式查询")
    @PostMapping("/card/info")
    public AnswerCardVo cardInfo(HttpServletRequest request, @RequestParam Long examId,
                                 @RequestParam Integer number) {
        ExamEntity exam = examService.getById(examId);
        if (exam == null || !exam.getEnable()) {
            throw ParameterExceptions.EXAM_NOT_FOUND;
        }

        AnswerCardEntity card = answerCardService.findByExamAndNumber(examId, number);
        if (card == null) {
            throw new ParameterException("找不到卡格式");
        }

        AnswerCardVo vo = new AnswerCardVo(card);
        User user = getAccessUser();
        if (user.getRole().equals(Role.SCANNER)) {
            vo.setAdapteFile(adapteFileService.listByExamIdAndCardNumberForScanner(examId, number,
                    RequestUtil.getIpAddress(request)));
        } else {
            vo.setAdapteFile(adapteFileService.listByExamIdAndCardNumber(examId, card.getNumber()));
        }
        vo.setSubjectList(answerCardSubjectService.findByExamAndNumber(examId, card.getNumber()));
        return vo;
    }

}
