package cn.com.qmth.scancentral.controller.admin;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.boot.core.collection.PageResult;

import cn.com.qmth.scancentral.bean.User;
import cn.com.qmth.scancentral.controller.BaseController;
import cn.com.qmth.scancentral.service.ExamService;
import cn.com.qmth.scancentral.service.StudentService;
import cn.com.qmth.scancentral.vo.answerquery.AnswerQueryVo;
import cn.com.qmth.scancentral.vo.assginedcheck.AssginedTaskResult;
import cn.com.qmth.scancentral.vo.assginedcheck.AssignedTaskSaveVo;
import cn.com.qmth.scancentral.vo.auditor.AuditorOverview;
import cn.com.qmth.scancentral.vo.task.TaskStatusVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

@RestController
@Api(tags = "审核员管理接口")
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/auditor")
@Aac(strict = false, auth = true)
public class AuditorController extends BaseController {

    @Autowired
    private ExamService examService;

    @Autowired
    private StudentService studentService;

    @ApiOperation(value = "考试概览数据")
    @RequestMapping(value = "/exam/overview", method = RequestMethod.POST)
    public AuditorOverview overview(@RequestParam Long examId) {
        User user = getAccessUser();
        return examService.getAuditorOverview(examId, user);
    }

    @ApiOperation(value = "审核员复核校验任务获取")
    @RequestMapping(value = "/check/assigned/task/get", method = RequestMethod.POST)
    public AnswerQueryVo get(@RequestParam Long examId,@RequestParam String subjectCode) {
        User user = getAccessUser();
        return studentService.getAssignedCheckTask(examId,subjectCode, user);
    }

    @ApiOperation(value = "审核员复核校验任务历史")
    @RequestMapping(value = "/check/assigned/task/history", method = RequestMethod.POST)
    public PageResult<AnswerQueryVo> history(@RequestParam Long examId,@RequestParam String subjectCode, @RequestParam Long pageSize, @RequestParam Long pageNumber) {
        User user = getAccessUser();
        return studentService.getAssignedCheckTaskHistory(examId,subjectCode, pageNumber,pageSize, user);
    }

    @ApiOperation(value = "审核员复核校验任务提交")
    @RequestMapping(value = "/check/assigned/task/save", method = RequestMethod.POST)
    public AssignedTaskSaveVo submit(@Validated @RequestBody AssginedTaskResult result) {
        User user = getAccessUser();
        return studentService.submitAssignedCheckTask(result, user);
    }

    @ApiOperation(value = "审核员复核校验任务释放")
    @PostMapping("/check/assigned/task/release")
    public Object release(@RequestParam Long examId,@RequestParam String subjectCode) {
        User user = getAccessUser();
        studentService.releaseAssignedCheckTaskByUser(examId,subjectCode, user.getAccount());
        return success(true);
    }

    @ApiOperation(value = "审核员复核校验任务状态")
    @PostMapping("/check/assigned/task/status")
    public TaskStatusVo statusVo(@RequestParam Long examId,@RequestParam String subjectCode) {
        User user = getAccessUser();
        return studentService.getAssignedCheckTaskStatus(examId,subjectCode, user);
    }


}