package cn.com.qmth.scancentral.controller.admin;

import javax.annotation.Resource;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.boot.core.collection.PageResult;
import com.qmth.boot.core.concurrent.service.ConcurrentService;
import com.qmth.boot.core.exception.ParameterException;
import com.qmth.boot.core.exception.ReentrantException;

import cn.com.qmth.scancentral.controller.BaseController;
import cn.com.qmth.scancentral.enums.LockType;
import cn.com.qmth.scancentral.service.AnswerCardService;
import cn.com.qmth.scancentral.vo.ResultVo;
import cn.com.qmth.scancentral.vo.card.CardEdit;
import cn.com.qmth.scancentral.vo.card.CardInfo;
import cn.com.qmth.scancentral.vo.card.CardQuery;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

@RestController
@Api(tags = "卡格式相关接口")
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/admin/card")
@Aac(strict = false, auth = true)
public class CardController extends BaseController {

    protected static final Logger log = LoggerFactory.getLogger(CardController.class);

    @Autowired
    private AnswerCardService answerCardService;

    @Resource
    private ConcurrentService concurrentService;

    @ApiOperation(value = "查询卡格式列表（分页）")
    @RequestMapping(value = "page", method = RequestMethod.POST)
    public PageResult<CardInfo> list(CardQuery query) {
        return answerCardService.pageQuery(query);
    }

    @ApiOperation(value = "删除卡格式")
    @RequestMapping(value = "/delete", method = RequestMethod.POST)
    public ResultVo answerDelete(@RequestParam Long examId, @RequestParam Integer number) {
        if (concurrentService.getLock(LockType.CARD_SYNC + "-" + examId).tryLock()) {
            try {
                answerCardService.delete(getAccessUser(), examId, number);
                return new ResultVo(System.currentTimeMillis());
            } finally {
                concurrentService.getLock(LockType.CARD_SYNC + "-" + examId).unlock();
            }
        } else {
            throw new ReentrantException("正在同步卡格式，请稍后再试");
        }
    }

    @ApiOperation(value = "导入卡格式")
    @RequestMapping(value = "/import", method = RequestMethod.POST)
    public ResultVo cardImport(CardEdit info, @RequestParam MultipartFile file) {
        if (info.getExamId() == null) {
            throw new ParameterException("考试ID不能为空");
        }
        if (concurrentService.getLock(LockType.CARD_SYNC + "-" + info.getExamId()).tryLock()) {
            try {
                answerCardService.saveCard(info, file);
                return new ResultVo(System.currentTimeMillis());
            } finally {
                concurrentService.getLock(LockType.CARD_SYNC + "-" + info.getExamId()).unlock();
            }
        } else {
            throw new ReentrantException("正在同步卡格式，请稍后再试");
        }
    }

    // @ApiOperation(value = "答题卡适配卡格式上传")
    // @RequestMapping(value = "/answer/adapte/upload", method =
    // RequestMethod.POST)
    // public UriVo adapteUpload(HttpServletRequest request, @RequestParam Long
    // examId, @RequestParam Integer cardNumber,
    // @RequestParam MultipartFile file, @RequestParam String md5, @RequestParam
    // Integer dpi) {
    // return adapteFileService.save(RequestUtil.getIpAddress(request),
    // getAccessUser().getRole(), examId, cardNumber,
    // md5, dpi, file);
    // }
    //
    // @ApiOperation(value = "卡格式详情")
    // @RequestMapping(value = "/info", method = RequestMethod.POST)
    // public Map<String, Object> info(@RequestParam Long examId) {
    // Map<String, Object> status = new HashMap<String, Object>();
    // status.put("synching", concurrentService.isLocked(LockType.CARD_SYNC +
    // "-" + examId));
    // status.put("updateTime", examService.getById(examId).getCardSyncTime());
    // Map<String, Object> result = new HashMap<String, Object>();
    // result.put("status", status);
    // result.put("answer", answerCardService.listByExamId(examId));
    // result.put("package", new ArrayList<>());
    // return result;
    // }
    //
    // @ApiOperation(value = "创建/修改签到表卡格式")
    // @RequestMapping(value = "/package/save", method = RequestMethod.POST)
    // public JSONObject packageSave() {
    // JSONObject result = new JSONObject();
    // return result;
    // }

}
