package cn.com.qmth.scancentral.controller.admin;

import cn.com.qmth.scancentral.bean.User;
import cn.com.qmth.scancentral.controller.BaseController;
import cn.com.qmth.scancentral.service.OmrTaskService;
import cn.com.qmth.scancentral.vo.task.TaskResultVo;
import cn.com.qmth.scancentral.vo.task.TaskSaveVo;
import cn.com.qmth.scancentral.vo.task.TaskStatusVo;
import cn.com.qmth.scancentral.vo.task.TaskVo;
import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.constant.ApiConstant;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

@RestController
@Api(tags = "识别对照仲裁任务接口")
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/admin/check/omr/arbitrate")
@Aac(strict = false, auth = true)
public class OmrArbitrateController extends BaseController {

    protected static final Logger log = LoggerFactory.getLogger(OmrArbitrateController.class);

    @Autowired
    private OmrTaskService omrTaskService;

    @ApiOperation(value = "识别对照仲裁任务获取")
    @RequestMapping(value = "/get", method = RequestMethod.POST)
    public TaskVo get(@RequestParam Long groupId) {
        User user = getAccessUser();
        return omrTaskService.getArbitrateTask(groupId, user.getAccount());
    }

    @ApiOperation(value = "识别对照仲裁结果提交")
    @RequestMapping(value = "/save", method = RequestMethod.POST)
    public TaskSaveVo save(@RequestBody TaskResultVo result) {
        User user = getAccessUser();
        return omrTaskService.submitTask(result, user);
    }

    @ApiOperation(value = "识别对照仲裁进度状态")
    @RequestMapping(value = "/status", method = RequestMethod.POST)
    public TaskStatusVo status(@RequestParam Long groupId) {
        User user = getAccessUser();
        return omrTaskService.getStatus(groupId, true, user.getAccount());
    }

    @ApiOperation(value = "识别对照仲裁任务释放")
    @RequestMapping(value = "/release", method = RequestMethod.POST)
    public Object release(@RequestParam Long groupId) {
        User user = getAccessUser();
        omrTaskService.releaseByUser(groupId, user.getAccount());
        return success(true);
    }

    @ApiOperation(value = "识别对照仲裁任务历史")
    @RequestMapping(value = "/history", method = RequestMethod.POST)
    public TaskVo history(@RequestParam Long groupId, @RequestParam(required = false) Long id,
                          @RequestParam(required = false) Boolean next) {
        User user = getAccessUser();
        return omrTaskService.getArbitrateHistory(groupId, id, user.getAccount(), next);
    }

}
