package cn.com.qmth.scancentral.controller.admin;

import java.io.IOException;
import java.net.URLEncoder;
import java.util.Date;
import java.util.List;

import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.boot.tools.excel.ExcelWriter;
import com.qmth.boot.tools.excel.enums.ExcelType;

import cn.com.qmth.scancentral.bean.CardAnswerDomain;
import cn.com.qmth.scancentral.bean.WorkloadDomain;
import cn.com.qmth.scancentral.controller.BaseController;
import cn.com.qmth.scancentral.service.BatchService;
import cn.com.qmth.scancentral.service.ScannerCardService;
import cn.com.qmth.scancentral.service.ScannerService;
import cn.com.qmth.scancentral.service.SystemConfigService;
import cn.com.qmth.scancentral.util.DateUtil;
import cn.com.qmth.scancentral.vo.ScannerWorkloadVo;
import cn.com.qmth.scancentral.vo.UpdateCountVo;
import cn.com.qmth.scancentral.vo.scannerinfo.Control;
import cn.com.qmth.scancentral.vo.scannerinfo.ScannerInfoVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

@RestController
@Api(tags = "扫描员管理接口")
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/admin/scanner")
@Aac(strict = false, auth = true)
public class ScannerController extends BaseController {

    @Autowired
    private ScannerService scannerService;

    @Autowired
    private ScannerCardService scannerCardService;

    @Autowired
    private BatchService batchService;

    @Autowired
    private SystemConfigService systemConfigService;

    @ApiOperation(value = "扫描员管理详情")
    @PostMapping("info")
    public ScannerInfoVo scannerInfo(@RequestParam Long examId) {
        return scannerService.scannerInfo(examId);
    }

    @ApiOperation(value = "扫描员工作量统计")
    @PostMapping("workload")
    public List<ScannerWorkloadVo> workload(WorkloadDomain domain) {
        return batchService.workload(domain);
    }

    @ApiOperation(value = "修改扫描员登陆控制")
    @PostMapping("control")
    public Control control(Control control) {
        return systemConfigService.control(control);
    }

    @ApiOperation(value = "修改扫描员绑定卡格式")
    @PostMapping("card/answer")
    public UpdateCountVo cardAnswer(CardAnswerDomain domain) {
        return scannerCardService.cardAnswer(domain);
    }

    @ApiOperation(value = "扫描员工作量统计导出")
    @PostMapping("workload/export")
    public void workloadExport(WorkloadDomain domain, HttpServletResponse response) throws IOException {
        String fileName = URLEncoder.encode("扫描员工作量统计", "UTF-8");
        response.setHeader("Content-Disposition", "inline; filename=" + fileName + ".xlsx");
        response.setContentType("application/vnd.ms-excel");
        ExcelWriter writer = ExcelWriter.create(ExcelType.XLSX);
        // PageListIterator<ScannerWorkloadVo> iterator = new
        // PageListIterator<ScannerWorkloadVo>(100) {
        //
        // @Override
        // public Collection<ScannerWorkloadVo> getPageList(int pageNumber, int
        // pageSize) {
        // return batchService.workload(domain);
        // }
        // };
        List<ScannerWorkloadVo> ret = batchService.workload(domain);
        String title = getText(domain.getStartTime()) + "-" + getText(domain.getEndTime());
        String[] titles = new String[1];
        titles[0] = title;
        writer.writeObjects("扫描员工作量统计", titles, ScannerWorkloadVo.class, ret.iterator());
        writer.output(response.getOutputStream());
    }

    private String getText(Long time) {
        if (time == null) {
            return "";
        }
        return DateUtil.format(new Date(time), DateUtil.DatePatterns.CHINA_DEFAULT);
    }
}