package cn.com.qmth.scancentral.controller.admin;

import java.io.IOException;
import java.net.URLEncoder;
import java.util.Collection;

import javax.servlet.http.HttpServletResponse;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.boot.core.collection.PageResult;
import com.qmth.boot.tools.excel.ExcelWriter;
import com.qmth.boot.tools.excel.enums.ExcelType;
import com.qmth.boot.tools.iterator.PageListIterator;

import cn.com.qmth.scancentral.controller.BaseController;
import cn.com.qmth.scancentral.service.StudentService;
import cn.com.qmth.scancentral.util.ResouceUtil;
import cn.com.qmth.scancentral.vo.student.StudentAnswerVo;
import cn.com.qmth.scancentral.vo.student.StudentPageQuery;
import cn.com.qmth.scancentral.vo.student.StudentPageVo;
import cn.com.qmth.scancentral.vo.subject.TaskIdVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

@RestController("adminStudentController")
@Api(tags = "考生相关接口")
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/admin/student")
@Aac(strict = false, auth = true)
public class StudentController extends BaseController {

    @Autowired
    private StudentService studentService;

    @ApiOperation(value = "考生查询")
    @PostMapping("answer")
    public StudentAnswerVo studentAnswer(@RequestParam Long batchId, @RequestParam Long studentId) {
        return studentService.studentAnswer(batchId, studentId);
    }

    @ApiOperation(value = "分页查询考生")
    @PostMapping(value = "page")
    public PageResult<StudentPageVo> studentPage(@Validated StudentPageQuery query) {
        return studentService.studentPage(query);
    }

    @ApiOperation(value = "导出考生")
    @PostMapping(value = "export")
    public void studentExport(@Validated StudentPageQuery query, HttpServletResponse response) throws IOException {
        String fileName = URLEncoder.encode("考生信息", "UTF-8");
        response.setHeader("Content-Disposition", "inline; filename=" + fileName + ".xlsx");
        response.setContentType("application/vnd.ms-excel");
        ExcelWriter writer = ExcelWriter.create(ExcelType.XLSX);
        PageListIterator<StudentPageVo> iterator = new PageListIterator<StudentPageVo>(5000) {

            @Override
            public Collection<StudentPageVo> getPageList(int pageNumber, int pageSize) {
                query.setPageNumber(pageNumber);
                query.setPageSize(pageSize);
                return studentService.studentList(query);
            }
        };
        writer.writeObjects("考生信息", null, StudentPageVo.class, iterator);
        writer.output(response.getOutputStream());

    }

    @ApiOperation(value = "违纪导入模版下载")
    @PostMapping("breach/template")
    public void breachTemplate(HttpServletResponse response) {
        exportFile("违纪导入模板.txt", ResouceUtil.getStream("templates/breach-import.txt"));
    }

    @ApiOperation(value = "违纪导入")
    @PostMapping("breach/import")
    public TaskIdVo breachImport(@RequestParam Long examId, @RequestParam String subjectCode,
            @RequestParam MultipartFile file) {
        return studentService.breachImport(examId, subjectCode, file);
    }

    @ApiOperation(value = "考生状态导入模版下载")
    @PostMapping("cust-status/template")
    public void custStatusTemplate(HttpServletResponse response) {
        exportFile("考生状态导入模板.txt", ResouceUtil.getStream("templates/cust-status-import.txt"));
    }

    @ApiOperation(value = "考生状态导入")
    @PostMapping("cust-status/import")
    public TaskIdVo custStatusImport(@RequestParam Long examId, @RequestParam String subjectCode,
            @RequestParam MultipartFile file) {
        return studentService.custStatusImport(examId, subjectCode, file);
    }
}
