package cn.com.qmth.scancentral.controller.admin;

import cn.com.qmth.scancentral.controller.BaseController;
import cn.com.qmth.scancentral.entity.ExamEntity;
import cn.com.qmth.scancentral.service.ExamService;
import cn.com.qmth.scancentral.service.StudentImportService;
import cn.com.qmth.scancentral.service.StudentService;
import cn.com.qmth.scancentral.util.FileUtil;
import cn.com.qmth.scancentral.util.ResouceUtil;
import cn.com.qmth.scancentral.vo.ResultVo;
import cn.com.qmth.scancentral.vo.studentimport.ImportTaskVo;
import cn.com.qmth.scancentral.vo.studentimport.StudentCountVo;
import cn.com.qmth.scancentral.vo.studentimport.StudentImportConfigVo;
import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.boot.core.exception.StatusException;
import com.qmth.boot.tools.uuid.FastUUID;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.io.FileUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
@Api(tags = "考生导入相关接口")
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/admin/student")
@Aac(strict = false, auth = true)
public class StudentImportController extends BaseController {

    private static final Logger log = LoggerFactory.getLogger(StudentImportController.class);

    @Autowired
    private StudentService studentService;

    @Autowired
    private StudentImportService studentImportService;

    @Autowired
    private ExamService examService;

    @ApiOperation(value = "考生导入")
    @PostMapping("/import")
    public Map<String, Object> studentImport(@RequestParam Long examId,
                                             @RequestParam(required = false) String subjectCode,
                                             @RequestParam MultipartFile file) throws Exception {
        if (studentImportService.existRunningStudentImportTask()) {
            throw new StatusException("有正在运行的考生导入任务，请稍后再试！");
        }

        ExamEntity exam = examService.getById(examId);
        if (exam == null) {
            throw new StatusException("当前考试不存在！");
        }

        if (exam.getYear() == null || exam.getYearHalf() == null) {
            throw new StatusException("未设置考试年度或考次！");
        }

        String fileSuffix = FileUtil.getFileSuffix(file.getOriginalFilename());
        if (!".txt".equals(fileSuffix)) {
            throw new StatusException("导入模板目前仅支持后缀名为“.txt”的文件！");
        }

        final String taskId = FastUUID.get();
        List<String> lines;
        try {
            final String tempDir = "temp/import_data/";
            FileUtil.makeDirs(tempDir);
            File dataFile = new File(tempDir + taskId + fileSuffix);
            FileUtils.writeByteArrayToFile(dataFile, file.getBytes());
            lines = FileUtil.readAllLines(dataFile);
            log.warn("【考生导入】读取考生数据文件共{}行! taskId:{}", lines.size(), taskId);
        } catch (Exception e) {
            log.error(e.getMessage(), e);
            throw new StatusException("读取导入数据文件失败！");
        }

        studentImportService.studentImport(taskId, lines, examId, subjectCode, exam.getYear(), exam.getYearHalf());

        Map<String, Object> result = new HashMap<>();
        result.put("taskId", taskId);
        return result;
    }

    @ApiOperation(value = "考生导入进度查询")
    @PostMapping("/import/progress")
    public ImportTaskVo studentImportProgress(@RequestParam String taskId) {
        return studentImportService.studentImportProgress(taskId);
    }

    @ApiOperation(value = "查询科目考生数")
    @PostMapping("/count")
    public List<StudentCountVo> countStudent(@RequestParam Long examId) {
        return studentService.countStudent(examId);
    }

    @ApiOperation(value = "考生清空")
    @PostMapping(value = "/clear")
    public ResultVo studentClean(@RequestParam Long examId, @RequestParam String subjectCode) {
        studentService.studentClean(examId, subjectCode);
        return new ResultVo(System.currentTimeMillis());
    }

    @ApiOperation(value = "获取考生导入参数设置")
    @PostMapping(value = "/import/config")
    public StudentImportConfigVo getStudentImportConfig(@RequestParam Long examId) {
        return examService.getStudentImportConfig(examId);
    }

    @ApiOperation(value = "保存考生导入参数设置")
    @PostMapping(value = "/import/config/save")
    public ResultVo saveStudentImportConfig(StudentImportConfigVo config) {
        examService.saveStudentImportConfig(config);
        return new ResultVo(System.currentTimeMillis());
    }

    @ApiOperation(value = "考生导入模版下载")
    @GetMapping("/import/template")
    public void importTemplate() {
        super.exportFile("考生导入模板.txt", ResouceUtil.getStream("templates/student-import.txt"));
    }

}
