package cn.com.qmth.scancentral.controller.admin;

import cn.com.qmth.scancentral.bean.User;
import cn.com.qmth.scancentral.controller.BaseController;
import cn.com.qmth.scancentral.entity.UserEntity;
import cn.com.qmth.scancentral.service.UserService;
import cn.com.qmth.scancentral.vo.ResultVo;
import cn.com.qmth.scancentral.vo.user.UserEdit;
import cn.com.qmth.scancentral.vo.user.UserQuery;
import cn.com.qmth.scancentral.vo.user.UserVo;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.boot.core.collection.PageResult;
import com.qmth.boot.core.exception.ParameterException;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

@RestController
@Api(tags = "用户相关接口")
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/admin/user")
@Aac(strict = false, auth = true)
public class UserController extends BaseController {

    @Autowired
    private UserService userService;

    @ApiOperation(value = "获取用户列表")
    @RequestMapping(value = "/page", method = RequestMethod.POST)
    public PageResult<UserVo> page(UserQuery query) {
        User user = getAccessUser();
        return userService.findUserPage(query, user);
    }

    @ApiOperation(value = "新建、修改用户")
    @RequestMapping(value = "/save", method = RequestMethod.POST)
    public ResultVo save(UserEdit user) {
        user.setOperateUserId(getAccessUser().getId());
        userService.saveUser(user);
        return new ResultVo(System.currentTimeMillis());
    }

    @ApiOperation(value = "启用、禁用用户")
    @RequestMapping(value = "/toggle", method = RequestMethod.POST)
    public ResultVo updateEnable(@RequestParam Long userId, @RequestParam Boolean enable) {
        User user = getAccessUser();
        UserEntity userEntity = userService.getById(userId);
        if (userEntity == null) {
            throw new ParameterException("用户不存在");
        }

        LambdaUpdateWrapper<UserEntity> updateWrapper = new LambdaUpdateWrapper<>();
        updateWrapper.set(UserEntity::getEnable, enable);
        updateWrapper.set(UserEntity::getUpdateTime, System.currentTimeMillis());
        updateWrapper.set(UserEntity::getUpdaterId, user.getId());
        updateWrapper.eq(UserEntity::getId, userEntity.getId());
        userService.update(updateWrapper);

        return new ResultVo(System.currentTimeMillis());
    }

    @ApiOperation(value = "更新用户密码")
    @RequestMapping(value = "/password/reset", method = RequestMethod.POST)
    public ResultVo updatePassword(@RequestParam Long userId, @RequestParam String password) {
        User user = getAccessUser();
        UserEntity userEntity = userService.getById(userId);
        if (userEntity == null) {
            throw new ParameterException("用户不存在");
        }

        LambdaUpdateWrapper<UserEntity> updateWrapper = new LambdaUpdateWrapper<>();
        updateWrapper.set(UserEntity::getPassword, password);
        updateWrapper.set(UserEntity::getUpdateTime, System.currentTimeMillis());
        updateWrapper.set(UserEntity::getUpdaterId, user.getId());
        updateWrapper.eq(UserEntity::getId, userEntity.getId());
        userService.update(updateWrapper);

        return new ResultVo(System.currentTimeMillis());
    }

}
