package cn.com.qmth.scancentral.controller.scan;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.boot.core.exception.ParameterException;

import cn.com.qmth.scancentral.controller.BaseController;
import cn.com.qmth.scancentral.entity.ExamEntity;
import cn.com.qmth.scancentral.entity.SubjectEntity;
import cn.com.qmth.scancentral.service.ExamService;
import cn.com.qmth.scancentral.service.SubjectService;
import cn.com.qmth.scancentral.vo.EnableVo;
import cn.com.qmth.scancentral.vo.scanexaminfo.ScanExamInfoVo;
import cn.com.qmth.scancentral.vo.scanexamlist.ScanExamListVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

@RestController
@Api(tags = "扫描功能-考试接口")
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/scan/exam")
@Aac(strict = false, auth = true)
public class ScanExamController extends BaseController {

    @Autowired
    private ExamService examService;
    @Autowired
    private SubjectService subjectService;

    @ApiOperation(value = "考试详情")
    @RequestMapping(value = "/info", method = RequestMethod.POST)
    public ScanExamInfoVo info(@RequestParam Long id) {
        return examService.getScanExamInfo(id, getAccessUser());
    }

    @ApiOperation(value = "考试列表")
    @RequestMapping(value = "/list", method = RequestMethod.POST)
    public List<ScanExamListVo> list() {
        return examService.getScanExamList();
    }

    @ApiOperation(value = "科目列表")
    @RequestMapping(value = "/subject/list", method = RequestMethod.POST)
    public List<SubjectEntity> subjectList(@RequestParam Long examId) {
        return subjectService.listByExamId(examId);
    }

    @ApiOperation(value = "锁屏密码校验")
    @RequestMapping(value = "/assigned/verify/password", method = RequestMethod.POST)
    public EnableVo verifyPassword(@RequestParam Long examId,@RequestParam String password) {
        ExamEntity exam = examService.getById(examId);
        if(exam==null){
            throw new ParameterException("exam不能为空");
        }
        if(exam.getScannerAssignedVerifyPassword().equals(password)){
            return new EnableVo(true);
        }else{
            return new EnableVo(false);
        }
    }
}
