package cn.com.qmth.scancentral.controller.scan;

import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.constant.ApiConstant;

import cn.com.qmth.scancentral.bean.User;
import cn.com.qmth.scancentral.controller.BaseController;
import cn.com.qmth.scancentral.service.OmrGroupService;
import cn.com.qmth.scancentral.service.OmrTaskService;
import cn.com.qmth.scancentral.service.SubjectService;
import cn.com.qmth.scancentral.vo.OmrGroupVo;
import cn.com.qmth.scancentral.vo.omr.OmrSubjectVo;
import cn.com.qmth.scancentral.vo.task.TaskResultVo;
import cn.com.qmth.scancentral.vo.task.TaskSaveVo;
import cn.com.qmth.scancentral.vo.task.TaskStatusVo;
import cn.com.qmth.scancentral.vo.task.TaskVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

@RestController
@Api(tags = "识别对照任务接口")
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/scan/omr")
@Aac(strict = false, auth = true)
public class ScanOmrController extends BaseController {

    protected static final Logger log = LoggerFactory.getLogger(ScanOmrController.class);

    @Autowired
    private OmrTaskService omrTaskService;

    @Autowired
    private OmrGroupService groupService;

    @Autowired
    private SubjectService subjectService;

    @ApiOperation(value = "科目列表")
    @RequestMapping(value = "subject/list", method = RequestMethod.POST)
    public List<OmrSubjectVo> subjectList(@RequestParam Long examId) {
        return subjectService.listByExamIdForOmr(examId);
    }

    @ApiOperation(value = "识别对照分组获取")
    @RequestMapping(value = "/group/list", method = RequestMethod.POST)
    public List<OmrGroupVo> groupList(@RequestParam Long examId, @RequestParam String subjectCode) {
        return groupService.listByExamIdAndSubjectCode(examId, subjectCode);
    }

    @ApiOperation(value = "识别对照任务获取")
    @RequestMapping(value = "/task/get", method = RequestMethod.POST)
    public TaskVo get(@RequestParam Long groupId) {
        User user = getAccessUser();
        return omrTaskService.getTask(groupId, user.getAccount());
    }

    @ApiOperation(value = "识别对照任务提交")
    @RequestMapping(value = "/task/save", method = RequestMethod.POST)
    public TaskSaveVo save(@Validated @RequestBody TaskResultVo result) {
        User user = getAccessUser();
        return omrTaskService.submitTask(result, user);
    }

    @ApiOperation(value = "识别对照任务状态")
    @RequestMapping(value = "/task/status", method = RequestMethod.POST)
    public TaskStatusVo status(@RequestParam Long groupId) {
        User user = getAccessUser();
        return omrTaskService.getStatus(groupId, false, user.getAccount());
    }

    @ApiOperation(value = "识别对照任务状态")
    @RequestMapping(value = "/task/release", method = RequestMethod.POST)
    public Object release(@RequestParam Long groupId) {
        User user = getAccessUser();
        omrTaskService.releaseByUser(groupId, user.getAccount());
        return success(true);
    }

    @ApiOperation(value = "识别对照任务历史")
    @RequestMapping(value = "/task/history", method = RequestMethod.POST)
    public TaskVo history(@RequestParam Long groupId, @RequestParam(required = false) Long id,
            @RequestParam(required = false) Boolean next) {
        User user = getAccessUser();
        return omrTaskService.history(groupId, id, user.getAccount(), next);
    }

}
