package cn.com.qmth.scancentral.service.impl;

import java.util.List;

import org.apache.commons.lang.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.jeffreyning.mybatisplus.service.MppServiceImpl;
import com.qmth.boot.core.exception.ParameterException;

import cn.com.qmth.scancentral.dao.AnswerCardSubjectDao;
import cn.com.qmth.scancentral.entity.AnswerCardSubjectEntity;
import cn.com.qmth.scancentral.service.AnswerCardSubjectService;
import cn.com.qmth.scancentral.vo.SubjectVo;

@Service
public class AnswerCardSubjectServiceImpl extends MppServiceImpl<AnswerCardSubjectDao, AnswerCardSubjectEntity>
        implements AnswerCardSubjectService {

    protected static final Logger log = LoggerFactory.getLogger(AnswerCardSubjectServiceImpl.class);

    @Override
    public List<SubjectVo> findByExamAndNumber(Long examId, Integer number) {
        if (examId == null) {
            throw new ParameterException("examId不能为空");
        }
        if (number == null) {
            throw new ParameterException("cardnumber不能为空");
        }
        return this.baseMapper.selectSubjectList(examId, number);
    }

    @Override
    public AnswerCardSubjectEntity findByExamAndNumberAndSubject(Long examId, Integer number, String subjectCode) {
        if (examId == null) {
            throw new ParameterException("examId不能为空");
        }
        if (number == null) {
            throw new ParameterException("cardnumber不能为空");
        }
        if (StringUtils.isBlank(subjectCode)) {
            throw new ParameterException("subjectCode不能为空");
        }
        QueryWrapper<AnswerCardSubjectEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<AnswerCardSubjectEntity> lw = wrapper.lambda();
        lw.eq(AnswerCardSubjectEntity::getExamId, examId);
        lw.eq(AnswerCardSubjectEntity::getCardNumber, number);
        lw.eq(AnswerCardSubjectEntity::getSubjectCode, subjectCode);
        return baseMapper.selectOne(wrapper);
    }

    @Transactional
    @Override
    public int deleteByExamAndNumber(Long examId, Integer number) {
        QueryWrapper<AnswerCardSubjectEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<AnswerCardSubjectEntity> lw = wrapper.lambda();
        lw.eq(AnswerCardSubjectEntity::getExamId, examId);
        lw.eq(AnswerCardSubjectEntity::getCardNumber, number);
        return baseMapper.delete(wrapper);
    }

    @Override
    public boolean checkSubject(Long examId, Integer number, String subjectCode) {
        AnswerCardSubjectEntity a = this.findByExamAndNumberAndSubject(examId, number, subjectCode);
        if (a != null) {
            return true;
        }
        QueryWrapper<AnswerCardSubjectEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<AnswerCardSubjectEntity> lw = wrapper.lambda();
        lw.eq(AnswerCardSubjectEntity::getExamId, examId);
        lw.eq(AnswerCardSubjectEntity::getCardNumber, number);
        return this.count(wrapper) == 0;
    }

}
