package cn.com.qmth.scancentral.service.impl;

import cn.com.qmth.scancentral.dao.BatchPaperDao;
import cn.com.qmth.scancentral.entity.BatchEntity;
import cn.com.qmth.scancentral.entity.BatchPaperEntity;
import cn.com.qmth.scancentral.entity.PaperEntity;
import cn.com.qmth.scancentral.service.BatchPaperService;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.jeffreyning.mybatisplus.service.MppServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Set;

@Service
public class BatchPaperServiceImpl extends MppServiceImpl<BatchPaperDao, BatchPaperEntity>
        implements BatchPaperService {

    @Override
    @Transactional
    public void update(BatchEntity batch, PaperEntity paper, Long studentId, Integer paperNumber, Boolean needCheck) {
        BatchPaperEntity batchPaperEntity = new BatchPaperEntity();
        batchPaperEntity.setBatchId(batch.getId());
        batchPaperEntity.setStudentId(studentId);
        batchPaperEntity.setPaperNumber(paperNumber);
        batchPaperEntity.setPaperId(paper.getId());
        batchPaperEntity.setCardNumber(paper.getCardNumber());
        batchPaperEntity.setAssigned(paper.getAssigned());
        batchPaperEntity.setNeedCheck(needCheck);
        saveOrUpdateByMultiId(batchPaperEntity);
    }

    @Override
    public List<BatchPaperEntity> findByBatchId(Long batchId) {
        QueryWrapper<BatchPaperEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<BatchPaperEntity> lw = wrapper.lambda();
        lw.eq(BatchPaperEntity::getBatchId, batchId);
        wrapper.orderByAsc("student_id", "paper_number");
        return this.list(wrapper);
    }

    @Override
    public BatchPaperEntity findByBatchIdAndStudentIdAndPaperNumber(Long batchId, Long studentId, Integer paperNumber) {
        QueryWrapper<BatchPaperEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<BatchPaperEntity> lw = wrapper.lambda();
        lw.eq(BatchPaperEntity::getBatchId, batchId);
        lw.eq(BatchPaperEntity::getStudentId, studentId);
        lw.eq(BatchPaperEntity::getPaperNumber, paperNumber);
        return baseMapper.selectOne(wrapper);
    }

    @Override
    public List<BatchPaperEntity> findByBatchIdAndStudentId(Long batchId, Long studentId) {
        QueryWrapper<BatchPaperEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<BatchPaperEntity> lw = wrapper.lambda();
        lw.eq(BatchPaperEntity::getBatchId, batchId);
        lw.eq(BatchPaperEntity::getStudentId, studentId);
        wrapper.orderByAsc("paper_number");
        return this.list(wrapper);
    }

    @Override
    public BatchPaperEntity findByPaperId(Long paperId) {
        QueryWrapper<BatchPaperEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<BatchPaperEntity> lw = wrapper.lambda();
        lw.eq(BatchPaperEntity::getPaperId, paperId);
        return baseMapper.selectOne(wrapper);
    }

    @Override
    public List<Long> findStudentIdByBatchId(Long batchId) {
        return baseMapper.findStudentIdByBatchId(batchId);
    }

    @Override
    public List<BatchPaperEntity> findByBatchIdAndStudentIds(Long batchId, Set<Long> studentIds) {
        QueryWrapper<BatchPaperEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<BatchPaperEntity> lw = wrapper.lambda();
        lw.eq(BatchPaperEntity::getBatchId, batchId);
        lw.in(BatchPaperEntity::getStudentId, studentIds);
        return this.list(wrapper);
    }

    @Override
    public Integer getScanStudentCount(Long examId, String subjectCode, long startTime, long endTime) {
        return baseMapper.getScanStudentCount(examId, subjectCode, startTime, endTime);
    }

}
