package cn.com.qmth.scancentral.service.impl;

import cn.com.qmth.scancentral.dao.ExamSummaryDao;
import cn.com.qmth.scancentral.entity.ExamEntity;
import cn.com.qmth.scancentral.entity.ExamSummaryEntity;
import cn.com.qmth.scancentral.service.*;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

@Service
public class ExamSummaryServiceImpl extends ServiceImpl<ExamSummaryDao, ExamSummaryEntity>
        implements ExamSummaryService {

    private static final Logger log = LoggerFactory.getLogger(ExamSummaryServiceImpl.class);

    @Autowired
    private ExamService examService;

    @Autowired
    private StudentService studentService;

    @Autowired
    private SubjectService subjectService;

    @Autowired
    private PaperService paperService;

    @Autowired
    private BatchService batchService;

    @Autowired
    private PackageResultService packageResultService;

    @Override
    @Transactional
    public ExamSummaryEntity find(Long examId) {
        // 独立模式下优先取定时计算结果
        // if (SystemMode.current() == SystemMode.STANDALONE) {
        ExamSummaryEntity es = getById(examId);
        if (es == null) {
            es = build(examId);
            saveOrUpdate(es);
        }
        return es;
        // }
        // // 云阅卷模式下每次实时计算
        // else {
        // return build(examId);
        // }
    }

    private ExamSummaryEntity build(Long examId) {
        ExamSummaryEntity es = new ExamSummaryEntity();
        es.setId(examId);
        es.updateScanAnswer(studentService.scanAnswerInfo(examId));
        es.setAnswerBatchCount(batchService.getValidCountByExam(examId));
        es.setAnswerMismatchCount(paperService.getMismatchCountByExam(examId));
        es.setSubjectCount(subjectService.countByExamId(examId));
        es.setPackageTotalCount(studentService.getPackageCountByExam(examId));
        es.setPackageScannedCount(packageResultService.getCountByExam(examId));
        es.setUploadProgress(studentService.getUploadProgress(examId));
        es.setStudentAnswerCount(studentService.getStudentAnswerCount(examId));
        return es;
    }

    // @Scheduled(fixedDelay = 20 * 1000, initialDelay = 20 * 1000)
    @Scheduled(cron = "0/60 * * * * ?")
    public void cronRefresh() {
        // 只有独立模式下开启定时刷新
        // if (SystemMode.MARKINGCLOUD.equals(SystemMode.current())) {
        // return;
        // }
        List<ExamEntity> list = examService.listEnable();
        for (ExamEntity exam : list) {
            log.info("ExamSummaryJob running for examId={}", exam.getId());
            saveOrUpdate(build(exam.getId()));
        }
    }
}
