package cn.com.qmth.scancentral.service.impl;

import cn.com.qmth.scancentral.enums.FileType;
import cn.com.qmth.scancentral.enums.FormatType;
import cn.com.qmth.scancentral.service.FileService;
import com.qmth.boot.core.exception.StatusException;
import com.qmth.boot.core.fss.store.FileStore;
import com.qmth.boot.tools.models.ByteArray;
import com.qmth.boot.tools.uuid.FastUUID;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.InputStream;

@Service
public class FileServiceImpl implements FileService {

    @Autowired
    private FileStore fileStore;

    @Override
    public String getAnswerCardUri(Long examId, Integer number) {
        return FileType.CARD.getPath(examId, number, FormatType.JSON.name().toLowerCase());
    }

    @Override
    public void downLoadAnswerCard(Long examId, Integer number, String md5, String url) throws Exception {
        ByteArray byteArray = ByteArray.fromUrl(url);
        fileStore.write(getAnswerCardUri(examId, number), new ByteArrayInputStream(byteArray.value()), md5);
    }

    @Override
    public String getSheetUri(Long examId, Long batchId, String examNumber, Integer paperNumber, Integer pageIndex) {
        return FileType.SHEET.getPath(examId, batchId, examNumber, paperNumber, pageIndex,
                FormatType.JPG.name().toLowerCase());
    }

    @Override
    public String getSliceUri(Long examId, Long batchId, String examNumber, Integer paperNumber, Integer pageIndex,
            Integer index) {
        return FileType.SLICE.getPath(examId, batchId, examNumber, paperNumber, pageIndex, index,
                FormatType.JPG.name().toLowerCase());
    }

    @Override
    public String getAdminSliceUri(Long examId, String examNumber, Integer paperNumber, Integer pageIndex,
            Integer index) {
        return FileType.ADMIN_SLICE.getPath(examId, examNumber, paperNumber, pageIndex, index, FastUUID.get(),
                FormatType.JPG.name().toLowerCase());
    }

    @Override
    public String getBreachUri(Long examId, String subjectCode) {
        return FileType.BREACH_FILE.getPath(examId, subjectCode);
    }

    @Override
    public String uploadBreach(InputStream in, String md5, Long examId, String subjectCode) {
        try {
            String path = getBreachUri(examId, subjectCode);
            fileStore.write(path, in, md5);
            return path;
        } catch (RuntimeException e) {
            throw e;
        } catch (Exception e) {
            throw new StatusException("文件上传出错", e);
        } finally {
            if (in != null) {
                try {
                    in.close();
                } catch (IOException e) {
                }
            }
        }
    }

    @Override
    public String getBreachInfoUri(Long examId, String subjectCode) {
        return FileType.BREACH_FILE.getPath(examId, subjectCode);
    }

    @Override
    public String uploadBreachInfo(InputStream in, String md5, Long examId, String subjectCode) {
        try {
            String path = getBreachInfoUri(examId, subjectCode);
            fileStore.write(path, in, md5);
            return path;
        } catch (RuntimeException e) {
            throw e;
        } catch (Exception e) {
            throw new StatusException("文件上传出错", e);
        } finally {
            if (in != null) {
                try {
                    in.close();
                } catch (IOException e) {
                }
            }
        }
    }

    @Override
    public String getCustStatusUri(Long examId, String subjectCode) {
        return FileType.CUST_STATUS_FILE.getPath(examId, subjectCode);
    }

    @Override
    public String uploadCustStatus(InputStream in, String md5, Long examId, String subjectCode) {
        try {
            String path = getCustStatusUri(examId, subjectCode);
            fileStore.write(path, in, md5);
            return path;
        } catch (RuntimeException e) {
            throw e;
        } catch (Exception e) {
            throw new StatusException("文件上传出错", e);
        } finally {
            if (in != null) {
                try {
                    in.close();
                } catch (IOException e) {
                }
            }
        }
    }

    @Override
    public String getCustStatusInfoUri(Long examId, String subjectCode) {
        return FileType.CUST_STATUS_INFO_FILE.getPath(examId, subjectCode);
    }

    @Override
    public String uploadCustStatusInfo(InputStream in, String md5, Long examId, String subjectCode) {
        try {
            String path = getCustStatusInfoUri(examId, subjectCode);
            fileStore.write(path, in, md5);
            return path;
        } catch (RuntimeException e) {
            throw e;
        } catch (Exception e) {
            throw new StatusException("文件上传出错", e);
        } finally {
            if (in != null) {
                try {
                    in.close();
                } catch (IOException e) {
                }
            }
        }
    }

    @Override
    public String getAdminSheetUri(Long examId, String examNumber, Integer paperNumber, Integer pageIndex) {
        return FileType.ADMIN_SHEET.getPath(examId, examNumber, paperNumber, pageIndex, FastUUID.get(),
                FormatType.JPG.name().toLowerCase());
    }

    @Override
    public String uploadAnswerCard(InputStream in, String md5, Long examId, Integer number) {
        try {
            String path = getAnswerCardUri(examId, number);
            fileStore.write(path, in, md5);
            return path;
        } catch (RuntimeException e) {
            throw e;
        } catch (Exception e) {
            throw new StatusException("文件上传出错", e);
        } finally {
            if (in != null) {
                try {
                    in.close();
                } catch (IOException e) {
                }
            }
        }
    }

    @Override
    public String uploadSheet(InputStream in, String md5, Long examId, Long batchId, String examNumber,
            Integer paperNumber, Integer pageIndex) {
        try {
            String path = getSheetUri(examId, batchId, examNumber, paperNumber, pageIndex);
            fileStore.write(path, in, md5);
            return path;
        } catch (RuntimeException e) {
            throw e;
        } catch (Exception e) {
            throw new StatusException("文件上传出错", e);
        } finally {
            if (in != null) {
                try {
                    in.close();
                } catch (IOException e) {
                }
            }
        }
    }

    @Override
    public String uploadSlice(InputStream in, String md5, Long examId, Long batchId, String examNumber,
            Integer paperNumber, Integer pageIndex, Integer index) {

        try {
            String path = getSliceUri(examId, batchId, examNumber, paperNumber, pageIndex, index);
            fileStore.write(path, in, md5);
            return path;
        } catch (RuntimeException e) {
            throw e;
        } catch (Exception e) {
            throw new StatusException("文件上传出错", e);
        } finally {
            if (in != null) {
                try {
                    in.close();
                } catch (IOException e) {
                }
            }
        }
    }

    @Override
    public String adminUploadSlice(InputStream in, String md5, Long examId, String examNumber, Integer paperNumber,
            Integer pageIndex, Integer index) {
        try {
            String path = getAdminSliceUri(examId, examNumber, paperNumber, pageIndex, index);
            fileStore.write(path, in, md5);
            return path;
        } catch (RuntimeException e) {
            throw e;
        } catch (Exception e) {
            throw new StatusException("文件上传出错", e);
        } finally {
            if (in != null) {
                try {
                    in.close();
                } catch (IOException e) {
                }
            }
        }
    }

    @Override
    public String adminUploadSheet(InputStream in, String md5, Long examId, String examNumber, Integer paperNumber,
            Integer pageIndex) {
        try {
            String path = getAdminSheetUri(examId, examNumber, paperNumber, pageIndex);
            fileStore.write(path, in, md5);
            return path;
        } catch (RuntimeException e) {
            throw e;
        } catch (Exception e) {
            throw new StatusException("文件上传出错", e);
        } finally {
            if (in != null) {
                try {
                    in.close();
                } catch (IOException e) {
                }
            }
        }
    }

    @Override
    public String uploadAdapteFile(InputStream in, String md5, Long examId, Integer number) {
        try {
            String path = getAdapteFileUri(examId, number);
            fileStore.write(path, in, md5);
            return path;
        } catch (RuntimeException e) {
            throw e;
        } catch (Exception e) {
            throw new StatusException("文件上传出错", e);
        } finally {
            if (in != null) {
                try {
                    in.close();
                } catch (IOException e) {
                }
            }
        }
    }

    @Override
    public String getAdapteFileUri(Long examId, Integer cardNumber) {
        return FileType.ADAPTE_FILE.getPath(examId, cardNumber, FastUUID.get(), FormatType.JSON.name().toLowerCase());
    }

}
