package cn.com.qmth.scancentral.service.impl;

import cn.com.qmth.scancentral.dao.PaperPageDao;
import cn.com.qmth.scancentral.entity.PaperPageEntity;
import cn.com.qmth.scancentral.service.PaperPageService;
import cn.com.qmth.scancentral.vo.paper.PaperPageCetVo;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.github.jeffreyning.mybatisplus.service.MppServiceImpl;
import com.qmth.boot.core.exception.ParameterException;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.validation.constraints.NotNull;
import java.util.List;

@Service
public class PaperPageServiceImpl extends MppServiceImpl<PaperPageDao, PaperPageEntity> implements PaperPageService {

    @Override
    @Transactional
    public void deleteByPaperId(Long paperId) {
        QueryWrapper<PaperPageEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<PaperPageEntity> lw = wrapper.lambda();
        lw.eq(PaperPageEntity::getPaperId, paperId);
        baseMapper.delete(wrapper);
    }

    @Override
    @Transactional
    public boolean updateSlicePath(@NotNull PaperPageEntity page) {
        LambdaUpdateWrapper<PaperPageEntity> lw = new LambdaUpdateWrapper<>();
        lw.set(PaperPageEntity::getSlicePath, page.getSlicePath());
        lw.eq(PaperPageEntity::getPaperId, page.getPaperId());
        lw.eq(PaperPageEntity::getPageIndex, page.getPageIndex());
        return update(lw);
    }

    @Override
    @Transactional
    public boolean updateSheetPath(@NotNull PaperPageEntity page) {
        LambdaUpdateWrapper<PaperPageEntity> lw = new LambdaUpdateWrapper<>();
        lw.set(PaperPageEntity::getSheetPath, page.getSheetPath());
        lw.eq(PaperPageEntity::getPaperId, page.getPaperId());
        lw.eq(PaperPageEntity::getPageIndex, page.getPageIndex());
        return update(lw);
    }

    @Override
    public List<PaperPageEntity> listByPaperId(Long paperId) {
        if (paperId == null) {
            throw new ParameterException("paperId不能为空");
        }
        QueryWrapper<PaperPageEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<PaperPageEntity> lw = wrapper.lambda();
        lw.eq(PaperPageEntity::getPaperId, paperId);
        wrapper.orderByAsc("page_index");
        return this.list(wrapper);
    }

    @Override
    public PaperPageEntity findPaperIdAndIndex(Long paperId, Integer index) {
        if (paperId == null) {
            throw new ParameterException("paperId不能为空");
        }
        if (index == null) {
            throw new ParameterException("index不能为空");
        }
        QueryWrapper<PaperPageEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<PaperPageEntity> lw = wrapper.lambda();
        lw.eq(PaperPageEntity::getPaperId, paperId);
        lw.eq(PaperPageEntity::getPageIndex, index);
        return baseMapper.selectOne(wrapper);
    }

    @Override
    public List<PaperPageEntity> listByPaperList(List<Long> paperIds) {
        QueryWrapper<PaperPageEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<PaperPageEntity> lw = wrapper.lambda();
        lw.in(PaperPageEntity::getPaperId, paperIds);
        wrapper.orderByAsc("paper_id", "page_index");
        return this.list(wrapper);
    }

    @Override
    public List<PaperPageCetVo> listByStudentIds(List<Long> studentIds) {
        return baseMapper.listByStudentIds(studentIds);
    }
}
