package cn.com.qmth.scancentral.service.impl;

import cn.com.qmth.scancentral.consumer.PictureCopyConsumer;
import cn.com.qmth.scancentral.service.PictureCopyService;
import cn.com.qmth.scancentral.service.StudentService;
import cn.com.qmth.scancentral.support.SpringContextHolder;
import cn.com.qmth.scancentral.vo.StudentUploadVo;
import org.apache.commons.collections4.CollectionUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.concurrent.*;

@Service
public class PictureCopyServiceImpl implements PictureCopyService {

    private static final Logger log = LoggerFactory.getLogger(PictureCopyService.class);

    @Autowired
    private StudentService studentService;

    @Override
    public void copy() {
        ExecutorService ex = null;
        try {
            int count = Math.min(5, Runtime.getRuntime().availableProcessors() * 2);
            ex = getExecutor(count);
            if (studentService.existPictureCopyData()) {
                for (;;) {
                    List<StudentUploadVo> dtos = findOneBatchData();
                    if (CollectionUtils.isEmpty(dtos)) {
                        // 拿不到数据，结束
                        break;
                    }
                    // 开始处理一轮
                    disposeBatch(ex, dtos);
                }
            }
        } catch (Exception e) {
            log.error("转存任务出错", e);
        } finally {
            if (ex != null) {
                ex.shutdown();
            }
        }
    }

    private void disposeBatch(ExecutorService ex, List<StudentUploadVo> dtos) throws InterruptedException {
        CountDownLatch endGate = new CountDownLatch(dtos.size());
        for (StudentUploadVo vo : dtos) {
            PictureCopyConsumer com = SpringContextHolder.getBean(PictureCopyConsumer.class);
            com.setVo(vo);
            com.setEndGate(endGate);
            ex.execute(com);
        }
        endGate.await();
    }

    private List<StudentUploadVo> findOneBatchData() {
        return studentService.findToPictureCopy(200);
    }

    private ExecutorService getExecutor(int threadCount) {
        ExecutorService exec = new ThreadPoolExecutor(threadCount, threadCount, 0L, TimeUnit.SECONDS,
                new LinkedBlockingQueue<>(threadCount * 2), r -> {
                    Thread t = new Thread(r);
                    return t;
                }, (r, executor) -> {
                    if (!executor.isShutdown()) {
                        try {
                            executor.getQueue().put(r);
                        } catch (InterruptedException e) {
                            throw new RuntimeException(e);
                        }
                    }
                });
        return exec;
    }
}
