package cn.com.qmth.scancentral.service.impl;

import cn.com.qmth.scancentral.bean.CardAnswerDomain;
import cn.com.qmth.scancentral.dao.ScannerCardDao;
import cn.com.qmth.scancentral.entity.ScannerCardEntity;
import cn.com.qmth.scancentral.service.AnswerCardService;
import cn.com.qmth.scancentral.service.ScannerCardService;
import cn.com.qmth.scancentral.service.SessionService;
import cn.com.qmth.scancentral.vo.UpdateCountVo;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.jeffreyning.mybatisplus.service.MppServiceImpl;
import com.qmth.boot.core.exception.ParameterException;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

@Service
public class ScannerCardServiceImpl extends MppServiceImpl<ScannerCardDao, ScannerCardEntity>
        implements ScannerCardService {

    @Autowired
    private SessionService sessionService;

    @Autowired
    private AnswerCardService answerCardService;

    @Override
    public int getCountByExamAndDevice(Long examId, String account) {
        QueryWrapper<ScannerCardEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<ScannerCardEntity> lw = wrapper.lambda();
        lw.eq(ScannerCardEntity::getExamId, examId);
        lw.eq(ScannerCardEntity::getDevice, account);
        return this.count(wrapper);
    }

    @Override
    public List<ScannerCardEntity> getListByExamAndDevice(Long examId, String account) {
        if (examId == null) {
            throw new ParameterException("examId不能为空");
        }
        if (account == null) {
            throw new ParameterException("account不能为空");
        }
        QueryWrapper<ScannerCardEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<ScannerCardEntity> lw = wrapper.lambda();
        lw.eq(ScannerCardEntity::getExamId, examId);
        lw.eq(ScannerCardEntity::getDevice, account);
        return this.list(wrapper);
    }

    private int deleteByExamAndDevice(Long examId, String device) {
        QueryWrapper<ScannerCardEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<ScannerCardEntity> lw = wrapper.lambda();
        lw.eq(ScannerCardEntity::getExamId, examId);
        lw.eq(ScannerCardEntity::getDevice, device);
        return baseMapper.delete(wrapper);
    }

    @Transactional
    @Override
    public int deleteByExamAndCardNumber(Long examId, Integer cardNumber) {
        QueryWrapper<ScannerCardEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<ScannerCardEntity> lw = wrapper.lambda();
        lw.eq(ScannerCardEntity::getExamId, examId);
        lw.eq(ScannerCardEntity::getCardNumber, cardNumber);
        return baseMapper.delete(wrapper);
    }

    @Transactional
    @Override
    public UpdateCountVo cardAnswer(CardAnswerDomain domain) {
        if (domain.getExamId() == null) {
            throw new ParameterException("ExamId不能为空");
        }
        if (domain.getCardNumber() == null) {
            throw new ParameterException("CardNumber不能为空");
        }
        if (CollectionUtils.isEmpty(domain.getDevice())) {
            throw new ParameterException("Device不能为空");
        }
        if (domain.getCardNumber() != 0
                && answerCardService.findByExamAndNumber(domain.getExamId(), domain.getCardNumber()) == null) {
            throw new ParameterException("卡格式未找到");
        }
        for (String device : domain.getDevice()) {
            if (sessionService.checkScannerOnline(device)) {
                throw new ParameterException("扫描员未离线：" + device);
            }
            if (domain.getCardNumber() != 0) {
                QueryWrapper<ScannerCardEntity> wrapper = new QueryWrapper<>();
                LambdaQueryWrapper<ScannerCardEntity> lw = wrapper.lambda();
                lw.eq(ScannerCardEntity::getExamId, domain.getExamId());
                lw.eq(ScannerCardEntity::getDevice, device);
                ScannerCardEntity sc = baseMapper.selectOne(wrapper);
                if (sc == null) {
                    sc = new ScannerCardEntity();
                    sc.setExamId(domain.getExamId());
                    sc.setDevice(device);
                }
                sc.setCardNumber(domain.getCardNumber());
                this.saveOrUpdateByMultiId(sc);
            } else {
                deleteByExamAndDevice(domain.getExamId(), device);
            }
        }
        UpdateCountVo vo = new UpdateCountVo();
        vo.setUpdateCount(domain.getDevice().size());
        return vo;
    }

}
