package cn.com.qmth.scancentral.service.impl;

import java.util.Date;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import cn.com.qmth.scancentral.bean.User;
import cn.com.qmth.scancentral.config.SysProperty;
import cn.com.qmth.scancentral.enums.Role;
import cn.com.qmth.scancentral.service.SessionService;

@Service
public class SessionServiceImpl implements SessionService {

    private final static Map<String, User> userSessionData = new ConcurrentHashMap<>();

    @Autowired
    private SysProperty sysProperty;

    @Override
    public Long updateUserSession(User user) {
        if (user.getSessionId() == null) {
            return null;
        }
        user.setActiveTime(new Date().getTime());
        userSessionData.put(user.getSessionId(), user);
        return user.getActiveTime();
    }

    @Override
    public void clearTimeOutUser() {
        Date d = new Date();
        Long now = d.getTime();
        for (String k : userSessionData.keySet()) {
            if (now - userSessionData.get(k).getActiveTime() > getTimeout()) {
                userSessionData.remove(k);
            }
        }
    }

    @Override
    public void userLogout(User user) {
        userSessionData.remove(user.getSessionId());
    }

    @Override
    public void userLogin(User user) {
        updateUserSession(user);
    }

    @Override
    public User getSessionUser(String sessionId) {
        return userSessionData.get(sessionId);
    }

    @Override
    public int getOnlineCount() {
        return userSessionData.size();
    }

    @Override
    public int getOnlineScannerCount() {
        int count = 0;
        for (User user : userSessionData.values()) {
            if (Role.SCANNER.equals(user.getRole())) {
                count++;
            }
        }
        return count;
    }

    @Override
    public boolean checkScannerOnline(String device) {
        String sessionId = Role.SCANNER.name() + "_" + device;
        return userSessionData.containsKey(sessionId);
    }

    private long getTimeout() {
        return (sysProperty.getHeartbeatTimeout() == null ? 600 : sysProperty.getHeartbeatTimeout()) * 1000L;
    }

    @Override
    public int getOnlineAuditorCount() {
        int count = 0;
        for (User user : userSessionData.values()) {
            if (Role.AUDITOR.equals(user.getRole())) {
                count++;
            }
        }
        return count;
    }

    @Override
    public boolean checkAuditorOnline(String loginName) {
        String sessionId = Role.AUDITOR.name() + "_" + loginName;
        return userSessionData.containsKey(sessionId);
    }

}
