package cn.com.qmth.scancentral.service.impl;

import cn.com.qmth.scancentral.dao.StudentPaperDao;
import cn.com.qmth.scancentral.entity.StudentPaperEntity;
import cn.com.qmth.scancentral.service.StudentPaperService;
import cn.com.qmth.scancentral.vo.StudentPaperCountVo;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.jeffreyning.mybatisplus.service.MppServiceImpl;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;

@Service
public class StudentPaperServiceImpl extends MppServiceImpl<StudentPaperDao, StudentPaperEntity>
        implements StudentPaperService {

    @Override
    public Integer countByStudentId(Long studentId) {
        QueryWrapper<StudentPaperEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<StudentPaperEntity> lw = wrapper.lambda();
        lw.eq(StudentPaperEntity::getStudentId, studentId);
        return baseMapper.selectCount(lw);
    }

    @Override
    public List<StudentPaperEntity> findPaperIdByExamId(Long examId) {
        return this.baseMapper.findPaperIdByExamId(examId);
    }

    @Override
    public StudentPaperEntity findByStudentIdAndPaperNumber(Long studentId, Integer paperNumber) {
        QueryWrapper<StudentPaperEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<StudentPaperEntity> lw = wrapper.lambda();
        lw.eq(StudentPaperEntity::getStudentId, studentId);
        lw.eq(StudentPaperEntity::getPaperNumber, paperNumber);
        return baseMapper.selectOne(wrapper);
    }

    @Override
    public List<StudentPaperEntity> findByStudentId(Long studentId) {
        QueryWrapper<StudentPaperEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<StudentPaperEntity> lw = wrapper.lambda();
        lw.eq(StudentPaperEntity::getStudentId, studentId);
        wrapper.orderByAsc("paper_number");
        return this.list(wrapper);
    }

    @Override
    public List<StudentPaperCountVo> findStudentPaperCount(List<Long> studentIds) {
        return baseMapper.findStudentPaperCount(studentIds);
    }

    @Transactional
    @Override
    public void removeByStudentId(Long studentId) {
        QueryWrapper<StudentPaperEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<StudentPaperEntity> lw = wrapper.lambda();
        lw.eq(StudentPaperEntity::getStudentId, studentId);
        this.baseMapper.delete(wrapper);
    }

    @Override
    public StudentPaperEntity findByPaperId(Long paperId) {
        QueryWrapper<StudentPaperEntity> wrapper = new QueryWrapper<>();
        LambdaQueryWrapper<StudentPaperEntity> lw = wrapper.lambda();
        lw.eq(StudentPaperEntity::getPaperId, paperId);
        return baseMapper.selectOne(wrapper);
    }

    @Override
    public List<Long> findStudentIdByExamId(Long examId, String subjectCode) {
        return this.baseMapper.findStudentIdByExamId(examId, subjectCode);
    }

}
