package cn.com.qmth.scancentral.service.impl;

import cn.com.qmth.scancentral.config.SysProperty;
import cn.com.qmth.scancentral.entity.*;
import cn.com.qmth.scancentral.enums.*;
import cn.com.qmth.scancentral.service.*;
import com.qmth.boot.core.concurrent.service.ConcurrentService;
import com.qmth.boot.core.exception.ParameterException;
import com.qmth.boot.core.exception.StatusException;
import com.qmth.boot.core.fss.store.FileStore;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.imageio.IIOImage;
import javax.imageio.ImageIO;
import javax.imageio.ImageWriteParam;
import javax.imageio.ImageWriter;
import javax.imageio.plugins.jpeg.JPEGImageWriteParam;
import java.awt.*;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.util.List;
import java.util.*;

@Service
public class ToolExportServiceImpl implements ToolExportService {

    private static final Logger log = LoggerFactory.getLogger(ToolExportServiceImpl.class);

    @Autowired
    private ConcurrentService concurrentService;

    @Autowired
    private FileStore fileStore;

    @Autowired
    private ExamService examService;

    @Autowired
    private PaperPageService paperPageService;

    @Autowired
    private StudentPaperService studentPaperService;

    @Autowired
    private StudentService studentService;

    // @Autowired
    // private SystemConfigService systemConfigService;

    @Autowired
    private SysProperty sysProperty;

    @Autowired
    private AnswerCardService answerCardService;

    @Autowired
    private ToolExportService toolExportService;

    private String getOwSheetUri(Long examId, String subjectCode, String examNumber, Integer paperNumber,
            Integer pageIndex) {
        return String.format("%s/%s/%s/%s/%s/%s-%s.jpg", "sheet", examId, subjectCode, examNumber.substring(0, 4),
                examNumber, paperNumber, pageIndex);
    }

    private String getOwSliceUri(Long examId, String subjectCode, String examNumber, Integer paperNumber,
            Integer pageIndex, Integer index) {
        return String.format("%s/%s/%s/%s/%s/%s-%s-%s.jpg", "slice", examId, subjectCode, examNumber.substring(0, 4),
                examNumber, paperNumber, pageIndex, index);
    }

    private String getCetSheetUri(Long examId, String examNumber, Integer pageTotalIndex) {
        return String.format("%s/%s/%s/%s/%s/%s/%s_%s.jpg", "sheet", examId, examNumber.substring(9, 10),
                examNumber.substring(0, 5), examNumber.substring(5, 6), examNumber.substring(10, 13), examNumber,
                pageTotalIndex);
    }

    @Override
    public String getCetSliceUri(Long examId, String examNumber, String sliceName) {
        return String.format("%s/%s/%s/%s/%s/%s/%s/%s-%s.jpg", "slice", examId, examNumber.substring(9, 10),
                examNumber.substring(0, 5), examNumber.substring(5, 6), examNumber.substring(10, 13), sliceName,
                examNumber, sliceName);
    }

    @Override
    public void studentCheck(Long studentId) {
        concurrentService.getReadWriteLock(LockType.STUDENT + "-" + studentId).writeLock().lock();
        StudentEntity student = null;
        try {
            student = studentService.getById(studentId);
            if (student == null) {
                return;
            }
            if (ScanStatus.UNEXIST.equals(student.getStatus())) {
                return;
            }
            if (student.getIncomplete()) {
                throw new ParameterException("考生扫描结果缺页");
            }
            if (UploadStatus.WAITING_UPLOAD.equals(student.getFileUploadStatus())) {
                return;
            }
            if (UploadStatus.ERROR.equals(student.getFileUploadStatus())) {
                throw new ParameterException("该考生图片转存任务出错");
            }
            toolExportService.studentCheckCet(student);
        } catch (Exception e) {
            if (student != null) {
                log.warn("图片检查异常！stuId:{} subjectCode:{} examNumber:{} err:{}", student.getId(),
                        student.getSubjectCode(), student.getExamNumber(), e.getMessage());
                studentService.updateImageCheckStatus(student.getId(), ImageCheckStatus.FAILED);
            } else {
                log.error("图片检查错误！err:{}", e.getMessage(), e);
            }
        } finally {
            concurrentService.getReadWriteLock(LockType.STUDENT + "-" + studentId).writeLock().unlock();
        }
    }

    @Override
    public void studentCheckCet(StudentEntity student) {
        List<StudentPaperEntity> spes = studentPaperService.findByStudentId(student.getId());
        if (CollectionUtils.isEmpty(spes)) {
            throw new ParameterException("没有扫描结果");
        }
        AnswerCardEntity card = answerCardService.findByExamAndNumber(student.getExamId(), student.getCardNumber());
        String imageTransferDir = sysProperty.getTransferDir();
        Map<String, List<String>> sliceMap = new LinkedHashMap<>();
        int index = 0;
        int pageIndex = 0;
        for (StudentPaperEntity spe : spes) {
            List<PaperPageEntity> pgs = paperPageService.listByPaperId(spe.getPaperId());
            if (CollectionUtils.isEmpty(pgs)) {
                throw new ParameterException("没有扫描页信息");
            }
            for (PaperPageEntity pg : pgs) {
                pageIndex++;
                String sheetPath = pg.getSheetPath();
                if (StringUtils.isBlank(sheetPath)) {
                    throw new ParameterException("没有原图");
                }
                String mirrorSheetPath = getCetSheetUri(student.getExamId(), student.getExamNumber(), pageIndex);
                File targetSheetFile = new File(imageTransferDir + "/" + mirrorSheetPath);
                if (!targetSheetFile.exists()) {
                    try {
                        FileUtils.copyToFile(fileStore.read(sheetPath), targetSheetFile);
                    } catch (Exception e) {
                        throw new StatusException("文件拷贝出错", e);
                    }
                }
                List<String> slices = pg.getSlicePath();
                if (CollectionUtils.isNotEmpty(slices)) {
                    for (String slice : slices) {
                        index++;
                        String sliceName = card.getSliceName().get(index - 1);
                        List<String> tem = sliceMap.get(sliceName);
                        if (tem == null) {
                            tem = new ArrayList<>();
                            sliceMap.put(sliceName, tem);
                        }
                        tem.add(slice);
                    }
                }
            }
        }
        for (String sliceName : sliceMap.keySet()) {
            String mirrorSlicePath = getCetSliceUri(student.getExamId(), student.getExamNumber(), sliceName);
            File targetSliceFile = new File(imageTransferDir + "/" + mirrorSlicePath);
            if (!targetSliceFile.exists()) {
                try {
                    joinImages(sliceMap.get(sliceName), targetSliceFile);
                } catch (Exception e) {
                    throw new StatusException("文件拷贝出错", e);
                }
            }
        }
    }

    @Override
    public void studentFileCopy(Long studentId, ImageTransferMode imageTransferMode) {
        // SystemConfigEntity config = systemConfigService.find();
        // if (config == null) {
        // throw new StatusException("未找到系统配置");
        // }
        // if (!SystemMode.STANDALONE.equals(SystemMode.current())) {
        // throw new StatusException("系统配置非独立模式");
        // }
        StudentEntity student = studentService.getById(studentId);
        if (student == null) {
            throw new StatusException("未找到考生");
        }
        if (ScanStatus.UNEXIST.equals(student.getStatus())) {
            throw new StatusException("考生还未扫描");
        }
        if (student.getIncomplete()) {
            throw new StatusException("考生扫描结果缺页");
        }
        ExamEntity exam = examService.getById(student.getExamId());
        if (exam.getImageTransferMode().equals(ImageTransferMode.OFF)) {
            return;
        }
        if (ImageTransferMode.CET.equals(exam.getImageTransferMode())) {
            studentFileCopyCet(student);
        } else if (ImageTransferMode.OW.equals(exam.getImageTransferMode())) {
            studentFileCopyOw(student);
        } else {
            throw new StatusException("转存模式配置错误");
        }
    }

    private void studentFileCopyCet(StudentEntity student) {
        List<StudentPaperEntity> spes = studentPaperService.findByStudentId(student.getId());
        if (CollectionUtils.isEmpty(spes)) {
            throw new StatusException("没有扫描结果");
        }
        AnswerCardEntity card = answerCardService.findByExamAndNumber(student.getExamId(), student.getCardNumber());
        String imageTransferDir = sysProperty.getTransferDir();
        Map<String, List<String>> sliceMap = new LinkedHashMap<>();
        int index = 0;
        int pageIndex = 0;
        for (StudentPaperEntity spe : spes) {
            List<PaperPageEntity> pgs = paperPageService.listByPaperId(spe.getPaperId());
            if (CollectionUtils.isEmpty(pgs)) {
                throw new StatusException("没有扫描页信息");
            }
            for (PaperPageEntity pg : pgs) {
                pageIndex++;
                String sheetPath = pg.getSheetPath();
                if (StringUtils.isBlank(sheetPath)) {
                    throw new StatusException("没有原图");
                }
                String mirrorSheetPath = getCetSheetUri(student.getExamId(), student.getExamNumber(), pageIndex);
                File targetSheetFile = new File(imageTransferDir + "/" + mirrorSheetPath);
                if (targetSheetFile.exists()) {
                    targetSheetFile.delete();
                }
                try {
                    FileUtils.copyToFile(fileStore.read(sheetPath), targetSheetFile);
                } catch (Exception e) {
                    throw new StatusException("文件拷贝出错", e);
                }
                List<String> slices = pg.getSlicePath();
                if (CollectionUtils.isNotEmpty(slices)) {
                    for (String slice : slices) {
                        index++;
                        String sliceName = card.getSliceName().get(index - 1);
                        List<String> tem = sliceMap.get(sliceName);
                        if (tem == null) {
                            tem = new ArrayList<>();
                            sliceMap.put(sliceName, tem);
                        }
                        tem.add(slice);
                    }
                }
            }
        }
        for (String sliceName : sliceMap.keySet()) {
            String mirrorSlicePath = getCetSliceUri(student.getExamId(), student.getExamNumber(), sliceName);
            File targetSliceFile = new File(imageTransferDir + "/" + mirrorSlicePath);
            if (targetSliceFile.exists()) {
                targetSliceFile.delete();
            }
            try {
                joinImages(sliceMap.get(sliceName), targetSliceFile);
            } catch (Exception e) {
                throw new StatusException("文件拷贝出错", e);
            }
            if (targetSliceFile.length() < 3 * 1024) {
                throw new StatusException("文件大小有误");
            }
        }
    }

    private void studentFileCopyOw(StudentEntity student) {
        List<StudentPaperEntity> spes = studentPaperService.findByStudentId(student.getId());
        if (CollectionUtils.isEmpty(spes)) {
            return;
        }
        String imageTransferDir = sysProperty.getTransferDir();
        int index = 0;
        for (StudentPaperEntity spe : spes) {
            List<PaperPageEntity> pgs = paperPageService.listByPaperId(spe.getPaperId());
            if (CollectionUtils.isEmpty(pgs)) {
                throw new StatusException("没有扫描页信息");
            }
            for (PaperPageEntity pg : pgs) {
                String sheetPath = pg.getSheetPath();
                if (StringUtils.isBlank(sheetPath)) {
                    throw new StatusException("没有原图");
                }
                String mirrorSheetPath = getOwSheetUri(student.getExamId(), student.getSubjectCode(),
                        student.getExamNumber(), spe.getPaperNumber(), pg.getPageIndex());
                File targetSheetFile = new File(imageTransferDir + "/" + mirrorSheetPath);
                if (targetSheetFile.exists()) {
                    targetSheetFile.delete();
                }
                try {
                    FileUtils.copyToFile(fileStore.read(sheetPath), targetSheetFile);
                } catch (Exception e) {
                    throw new StatusException("文件拷贝出错", e);
                }
                List<String> slices = pg.getSlicePath();
                if (CollectionUtils.isNotEmpty(slices)) {
                    for (String slice : slices) {
                        index++;
                        String mirrorSlicePath = getOwSliceUri(student.getExamId(), student.getSubjectCode(),
                                student.getExamNumber(), spe.getPaperNumber(), pg.getPageIndex(), index);
                        File targetSliceFile = new File(imageTransferDir + "/" + mirrorSlicePath);
                        if (targetSliceFile.exists()) {
                            targetSliceFile.delete();
                        }
                        try {
                            FileUtils.copyToFile(fileStore.read(slice), targetSliceFile);
                        } catch (Exception e) {
                            throw new StatusException("文件拷贝出错", e);
                        }
                    }
                }
            }
        }
    }

    private void joinImages(List<String> slicePath, File targetFile) throws Exception {
        boolean horizontal = false;
        List<BufferedImage> imageList = new ArrayList<>();
        for (String path : slicePath) {
            InputStream in = null;
            try {
                in = fileStore.read(path);
                BufferedImage image = ImageIO.read(in);
                imageList.add(image);
            } finally {
                in.close();
            }
        }
        int height = imageList.get(0).getHeight();
        int width = imageList.get(0).getWidth();
        if (horizontal) {
            height = imageList.stream().mapToInt(BufferedImage::getHeight).max().getAsInt();
            width = imageList.stream().mapToInt(BufferedImage::getWidth).sum();
        } else {
            width = imageList.stream().mapToInt(BufferedImage::getWidth).max().getAsInt();
            height = imageList.stream().mapToInt(BufferedImage::getHeight).sum();
        }
        // 创建拼接后的图片画布，参数分别为宽，高，类型，这里我们使用RGB3元色类型
        BufferedImage resultImage = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
        Graphics graphics = resultImage.getGraphics();
        int previousWidth = 0;
        int previousHeight = 0;
        for (BufferedImage image : imageList) {
            // 向画布上画图片
            graphics.drawImage(image, previousWidth, previousHeight, null);
            if (horizontal) {
                previousWidth += image.getWidth();
            } else {
                previousHeight += image.getHeight();
            }
        }
        if (targetFile.exists()) {
            targetFile.delete();
        }
        targetFile.getParentFile().mkdirs();
        FileOutputStream output = new FileOutputStream(targetFile);
        try {
            Iterator<ImageWriter> iter = ImageIO.getImageWritersByFormatName("JPEG");
            ImageWriter imageWriter = iter.next();
            JPEGImageWriteParam jpegParams = new JPEGImageWriteParam(Locale.getDefault());
            jpegParams.setCompressionMode(ImageWriteParam.MODE_EXPLICIT);
            jpegParams.setCompressionQuality(1.0F);
            imageWriter.setOutput(ImageIO.createImageOutputStream(output));
            imageWriter.write(null, new IIOImage(resultImage, null, null), jpegParams);
        } finally {
            output.close();
        }
    }

}
