package cn.com.qmth.scancentral.support;

import java.util.HashMap;
import java.util.Map;

public class TaskLockUtil {

    // 识别对照任务并发处理互斥锁
    private static final Map<String, TaskLock> omrTaskMap = new HashMap<>();

    // 实时审核任务并发处理互斥锁
    private static final Map<String, TaskLock> verifyTaskMap = new HashMap<>();

    // 图片检查任务并发处理互斥锁
    private static final Map<String, TaskLock> imageCheckTaskMap = new HashMap<>();

    // 数据检查任务并发处理互斥锁
    private static final Map<String, TaskLock> studentAnswerTaskMap = new HashMap<>();

    // 人工绑定任务并发处理互斥锁
    private static final Map<String, TaskLock> assignedCheckTaskMap = new HashMap<>();

    public static void clearTimeoutStudentAnswerTask(long timeoutMinute) {
        for (TaskLock taskLock : studentAnswerTaskMap.values()) {
            taskLock.expire(timeoutMinute);
        }
    }

    public static void clearTimeoutOmrTask(long timeoutMinute) {
        for (TaskLock taskLock : omrTaskMap.values()) {
            taskLock.expire(timeoutMinute);
        }
    }

    public static void clearTimeoutVerifyTask(long timeoutMinute) {
        for (TaskLock taskLock : verifyTaskMap.values()) {
            taskLock.expire(timeoutMinute);
        }
    }

    public static void clearTimeoutImageCheckTask(long timeoutMinute) {
        for (TaskLock taskLock : imageCheckTaskMap.values()) {
            taskLock.expire(timeoutMinute);
        }
    }

    public static void clearTimeoutAssignedCheckTaskTask(long timeoutMinute) {
        for (TaskLock taskLock : assignedCheckTaskMap.values()) {
            taskLock.expire(timeoutMinute);
        }
    }

    public static TaskLock getStudentAnswerTask(String key) {
        TaskLock taskLock = studentAnswerTaskMap.get(key);
        if (taskLock == null) {
            synchronized (studentAnswerTaskMap) {
                taskLock = studentAnswerTaskMap.computeIfAbsent(key, k -> new TaskLock());
            }
        }
        return taskLock;
    }

    public static TaskLock getOmrTask(String key) {
        TaskLock taskLock = omrTaskMap.get(key);
        if (taskLock == null) {
            synchronized (omrTaskMap) {
                taskLock = omrTaskMap.computeIfAbsent(key, k -> new TaskLock());
            }
        }
        return taskLock;
    }

    public static TaskLock getVerifyTask(String key) {
        TaskLock taskLock = verifyTaskMap.get(key);
        if (taskLock == null) {
            synchronized (verifyTaskMap) {
                taskLock = verifyTaskMap.computeIfAbsent(key, k -> new TaskLock());
            }
        }
        return taskLock;
    }

    public static TaskLock getImageCheckTask(String key) {
        TaskLock taskLock = imageCheckTaskMap.get(key);
        if (taskLock == null) {
            synchronized (imageCheckTaskMap) {
                taskLock = imageCheckTaskMap.computeIfAbsent(key, k -> new TaskLock());
            }
        }
        return taskLock;
    }

    public static TaskLock getAssignedCheckTask(String key) {
        TaskLock taskLock = assignedCheckTaskMap.get(key);
        if (taskLock == null) {
            synchronized (assignedCheckTaskMap) {
                taskLock = assignedCheckTaskMap.computeIfAbsent(key, k -> new TaskLock());
            }
        }
        return taskLock;
    }

}
