package cn.com.qmth.scancentral.task.thread;

import cn.com.qmth.scancentral.enums.LockType;
import cn.com.qmth.scancentral.service.OmrGroupService;
import com.qmth.boot.core.concurrent.model.Semaphore;
import com.qmth.boot.core.concurrent.service.ConcurrentService;
import com.qmth.boot.core.exception.StatusException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class OmrGroupBuildThread implements Runnable {

    protected static Logger log = LoggerFactory.getLogger(OmrGroupBuildThread.class);

    private Long groupId;

    private OmrGroupService groupService;

    private ConcurrentService concurrentService;

    private Semaphore semaphore;

    public OmrGroupBuildThread(Long groupId, OmrGroupService groupService, ConcurrentService concurrentService,
            Semaphore semaphore) {
        this.groupId = groupId;
        this.groupService = groupService;
        this.concurrentService = concurrentService;
        this.semaphore = semaphore;
    }

    @Override
    public void run() {
        try {
            log.info("OmrGroup[" + groupId + "] build task start");
            concurrentService.getReadWriteLock(LockType.OMR_GROUP + "-" + groupId).readLock().lock();
            groupService.buildTaskById(groupId);
            log.info("OmrGroup[" + groupId + "] build task finish");
        } catch (Exception e) {
            log.error("OmrGroup[" + groupId + "] build task error", e);
            throw new StatusException("识别对照分组生成任务失败");
        } finally {
            concurrentService.getReadWriteLock(LockType.OMR_GROUP + "-" + groupId).readLock().unlock();
            semaphore.release();
        }
    }

}
